


############################################################################
####   estimate rho for AR1 type spatial correlation on 1-dim locations ####
############################################################################

#---------------------------------------------------------------------------------------------------------------------
# Speicify Directories
#---------------------------------------------------------------------------------------------------------------------

  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v6" # version 6

#---------------------------------------------------------------------------------------------------------------------
# Load Packages and Functions
#---------------------------------------------------------------------------------------------------------------------

  setwd(workingpath)
  source("UF.SourceDir.r")
  SourceDir(Path = sourcepath, Trace = TRUE)  
  setwd(workingpath)
  batch <- 3
  corename <- paste("121209.AR1_1Dim_Separable_CorEst_AllTar_Fourier_batch", batch, sep = "")
  batchsize <- 50
  library(fda)
  library(sm)
  #sink(paste(corename, ".txt", sep = ""))

#---------------------------------------------------------------------------------------------------------------------
# Generate Simulated Data
#---------------------------------------------------------------------------------------------------------------------
  
  DistType <- "euclidean"
  NCurve <- 100
  Coordinate <- cbind(1 : NCurve, 0)
  Dim <- 1
  CorType <- "separable"
  AlphaList <- list(0, 0)
  DeltaList <- list(1, 1)
  KappaList <- list(0.5, 0.5)
  PhiList <- list(5, 2)
  TimeRange <- c(0, 1)
  MPerCurve <- 10
  SigmaVec <- c(0.2, 1)
  B <- batchsize
  K <- 3
  basis <- create.fourier.basis(TimeRange, nbasis = 3)
  MeanFD <- fd(rep(0, 3), basis)  
  EigenFunCoefMat <- matrix(0, K, K)
  EigenFunCoefMat[1:K, 1:K] <- diag(1, K)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Sign <- as.vector(sign(eval.fd(TimeRange[1]+0.01, EigenFun)))
  EigenFunCoefMat <- EigenFunCoefMat %*% diag(Sign)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Lambda <- exp(-(1:K)) * 10 
  NGrid <- 101
  Grids <- seq(TimeRange[1], TimeRange[2], length.out = NGrid)
  SimDataList <- SCorDatGen(DistType = DistType, Coordinate = Coordinate, CorType = CorType, KappaList = KappaList, 
                            PhiList = PhiList, TimeRange = TimeRange, MPerCurve = MPerCurve, SigmaVec = SigmaVec, B = B, MeanFD = MeanFD, 
                            EigenFun = EigenFun, Lambda = Lambda, TimeRegular = TRUE, Grids = Grids, AlphaList = AlphaList, DeltaList = DeltaList)
  SimObservation <- SimDataList$Observation
  SimTime <- SimDataList$Time  
  DistMat <- SimDataList$DistMat[[1]]

#---------------------------------------------------------------------------------------------------------------------
# Estimate Matern Correlation Parameters
#---------------------------------------------------------------------------------------------------------------------

  A <- length(PhiList)
  M <- length(MPerCurve)
  S <- length(SigmaVec)
  TimeRegular <- TRUE
  EvalTime.Regular <- Grids
  Rounding <- "closest"
  EvalTime.Mu <- EvalTime.Regular
  KAll <- c(2,3,4)
  EvalTime.Diag <- seq(0.3,0.7,length.out = 201)
  Rotate <- FALSE
  TargetType <- "polar"
  NTarget <- 20
  TargetCore <- 1 : NTarget
  TargetAll <- array(NA, dim = c(NTarget,2,2))
  TargetAll[,,1] <- cbind(TargetCore, TargetCore)
  TargetAll[,,2] <- cbind(rep(-91, NTarget), rep(91, NTarget))
  TargetIdxList <- list(1,1:5,1:10,1:15,1:20)
  TargetRep <- cbind(TargetCore,0)
  RmDiag <- FALSE
  Optim.Method <- c("BFGS","L-BFGS-B")
  NMethod <- length(Optim.Method)
  Log <- RepMat(t(c(FALSE,FALSE,FALSE,FALSE)),NMethod,1)
  MaxIt <- rep(1000,NMethod)
  DefaultPars <- RepMat(t(c(0,1,0.5,NA)),NMethod,1)
  InitialPars <- RepMat(t(c(NA,NA,NA,1)),NMethod,1)
  ParsBound <- array(NA, dim = c(NMethod,2,4))
  ParsBound[,1,] <- RepMat(t(c(NA,NA,NA,0.01)),NMethod,1)
  ParsBound[,2,] <- RepMat(t(c(NA,NA,NA,20)),NMethod,1)
  NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1)
  ReConstructType <- "separable"
  CVMean <- TRUE
  ModelSelection <- FALSE
  InvSym <- NULL

  PhiFitted.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod,4))
  Converge.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  PhiFitted.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod,4))
  Converge.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  CompTime <- array(NA, dim = c(B,A,S,M))
  EstimateList <- array(list(), dim = c(B,A,S,M))
  PhiErr.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  PhiErr.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Rho.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Rho.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  RhoErr.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  RhoErr.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  TrueRho <- c()

  for(b in 1 : B)
  {
    for(m in 1 : M)
    {
      for(s in 1 : S)
      {
        for(a in 1 : A)
        {
          now <- proc.time()[3] 
          thisObs <- SimObservation[[b,a,s,m]]
          thisTime <- SimTime[[b,a,s,m]]
          thisSigma <- SigmaVec[s]
          try(EstimateList[[b,a,s,m]] <- SCorEstMM(Y = thisObs, Time = thisTime, TimeRange = TimeRange, TimeRegular = TimeRegular, EvalTime.Regular = EvalTime.Regular, Rounding = Rounding, 
                                    EvalTime.Mu = EvalTime.Mu, KAll = KAll, EvalTime.Diag = EvalTime.Diag, Coordinate = Coordinate, Rotate = Rotate, InvSym = InvSym, RmDiag = RmDiag, 
                                    TargetType = TargetType, TargetAll = TargetAll, TargetIdxList = TargetIdxList, TargetRep = TargetRep, ModelSelection = ModelSelection, 
                                    DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, Log = Log, Optim.Method = Optim.Method, MaxIt = MaxIt, 
                                    NGridsSearch = NGridsSearch, Sigma = thisSigma, Dim = Dim, ReConstructType = ReConstructType, CVMean = CVMean), silent = TRUE)  
          if(!is.null(EstimateList[[b,a,s,m]]))
          {
            PhiFitted.Ind[b,a,s,m,,,,] <- EstimateList[[b,a,s,m]]$ParFitted.Ind
            Converge.Ind[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$Converge.Ind 
            PhiFitted.Proj[b,a,s,m,,,,] <- EstimateList[[b,a,s,m]]$ParFitted.Proj
            Converge.Proj[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$Converge.Proj
            for(t in 1 : NTarget)
            {
              for(k in 1 : max(KAll))
              {
                for(me in 1 : NMethod)
                {
                  PhiErr.Ind[b,a,s,m,t,k,me] <- (PhiFitted.Ind[b,a,s,m,t,k,me,4] - PhiList[[a]])^2
                  PhiErr.Proj[b,a,s,m,t,k,me] <- (PhiFitted.Proj[b,a,s,m,t,k,me,4] - PhiList[[a]])^2
                  Rho.Ind[b,a,s,m,t,k,me] <- matern(u = 1, kappa = 0.5, phi = PhiFitted.Ind[b,a,s,m,t,k,me,4])
                  Rho.Proj[b,a,s,m,t,k,me] <- matern(u = 1, kappa = 0.5, phi = PhiFitted.Proj[b,a,s,m,t,k,me,4])
                  TrueRho[a] <- matern(u = 1, kappa = 0.5, phi = PhiList[[a]])
                  RhoErr.Ind[b,a,s,m,t,k,me] <- (Rho.Ind[b,a,s,m,t,k,me] - TrueRho[a])^2
                  RhoErr.Proj[b,a,s,m,t,k,me] <- (Rho.Proj[b,a,s,m,t,k,me] - TrueRho[a])^2
                }
              }
            }
          }
          CompTime[b,a,s,m] <- proc.time()[3] - now
          print(paste("b =", b, "a =", a, "s =", s, "m =", m, "computing time is", CompTime[b,a,s,m]))

        }
      }
    }
  }
  
  #PhiErrMean.Ind <- apply(PhiErr.Ind, MARGIN = c(2,3,4,5,6,7), FUN = mean)
  #PhiErrMean.Proj <- apply(PhiErr.Proj, MARGIN = c(2,3,4,5,6,7), FUN = mean)
  #RhoErrMean.Ind <- apply(RhoErr.Ind, MARGIN = c(2,3,4,5,6,7), FUN = mean)
  #RhoErrMean.Proj <- apply(RhoErr.Proj, MARGIN = c(2,3,4,5,6,7), FUN = mean)
  
  #TarOptPhi.Ind <- apply(PhiErrMean.Ind, MARGIN = c(1,2,3,5,6), FUN = function(x){which(x == min(x))})
  #TarOptPhi.Proj <- apply(PhiErrMean.Proj, MARGIN = c(1,2,3,5,6), FUN = function(x){which(x == min(x))})
  #TarOptRho.Ind <- apply(RhoErrMean.Ind, MARGIN = c(1,2,3,5,6), FUN = function(x){which(x == min(x))})
  #TarOptRho.Proj <- apply(RhoErrMean.Proj, MARGIN = c(1,2,3,5,6), FUN = function(x){which(x == min(x))})

  save.image(paste(corename, ".RData", sep = ""))
  
  #sink()




 