


############################################################################
####   estimate rho for AR1 type spatial correlation on 1-dim locations ####
############################################################################

#---------------------------------------------------------------------------------------------------------------------
# Speicify Directories
#---------------------------------------------------------------------------------------------------------------------

  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v7" # version 7

#---------------------------------------------------------------------------------------------------------------------
# Load Packages and Functions
#---------------------------------------------------------------------------------------------------------------------

  setwd(workingpath)
  source("UF.SourceDir.r")
  SourceDir(Path = sourcepath, Trace = TRUE)  
  setwd(workingpath)
  batch <- 1
  corename <- paste("130108.AR1_1Dim_NonSeparable_CorEst_AllTar_Fourier_batch", batch, sep = "")
  batchsize <- 50
  library(fda)
  library(sm)
  #sink(paste(corename, ".txt", sep = ""))

#---------------------------------------------------------------------------------------------------------------------
# Generate Simulated Data
#---------------------------------------------------------------------------------------------------------------------
  
  DistType <- "euclidean"
  NCurve <- 100
  Coordinate <- cbind(1 : NCurve, 0)
  Dim <- 1
  CorType <- "diag"
  AlphaList <- list(c(0,0,0),c(0,0,0))
  DeltaList <- list(c(1,1,1),c(1,1,1))
  KappaList <- list(c(0.5,0.5,0.5),c(0.5,0.5,0.5))
  PhiList <- list(c(6,2,1),c(2,2,2))
  TimeRange <- c(0, 1)
  MPerCurve <- 10 
  SigmaVec <- c(0.5, 1)
  B <- batchsize
  K <- 3
  basis <- create.fourier.basis(TimeRange, nbasis = 3)
  MeanFD <- fd(rep(0, 3), basis)  
  EigenFunCoefMat <- matrix(0, K, K)
  EigenFunCoefMat[1:K, 1:K] <- diag(1, K)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Sign <- as.vector(sign(eval.fd(TimeRange[1]+0.01, EigenFun)))
  EigenFunCoefMat <- EigenFunCoefMat %*% diag(Sign)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Lambda <- exp(-(1:K)) * 10 
  NGrid <- 101
  Grids <- seq(TimeRange[1], TimeRange[2], length.out = NGrid)
  SimDataList <- SCorDatGen(DistType = DistType, Coordinate = Coordinate, CorType = CorType, KappaList = KappaList, 
                            PhiList = PhiList, TimeRange = TimeRange, MPerCurve = MPerCurve, SigmaVec = SigmaVec, B = B, MeanFD = MeanFD, 
                            EigenFun = EigenFun, Lambda = Lambda, TimeRegular = TRUE, Grids = Grids, AlphaList = AlphaList, DeltaList = DeltaList)
  SimObservation <- SimDataList$Observation
  SimTime <- SimDataList$Time  
  DistMat <- SimDataList$DistMat[[1]]

#---------------------------------------------------------------------------------------------------------------------
# Estimate Matern Correlation Parameters
#---------------------------------------------------------------------------------------------------------------------

  A <- length(PhiList)
  M <- length(MPerCurve)
  S <- length(SigmaVec)
  TimeRegular <- TRUE
  EvalTime.Regular <- Grids
  Rounding <- "closest"
  EvalTime.Mu <- EvalTime.Regular
  KAll <- c(3)
  EvalTime.Diag <- seq(0.3,0.7,length.out = 201)
  Rotate <- TRUE
  TargetType <- "polar"
  NTarget <- 20
  TargetCore <- 1 : NTarget
  TargetAll <- array(NA, dim = c(NTarget,2,2))
  TargetAll[,,1] <- cbind(TargetCore, TargetCore)
  TargetAll[,,2] <- cbind(rep(-91, NTarget), rep(91, NTarget))
  TargetIdxList <- list(1,1:5,1:10,1:15,1:20)
  TargetRep <- cbind(TargetCore,0)
  Optim.Method <- c("BFGS","L-BFGS-B")
  NMethod <- length(Optim.Method)
  Log <- RepMat(t(c(FALSE,FALSE,FALSE,FALSE)),NMethod,1)
  MaxIt <- rep(1000,NMethod)
  DefaultPars <- RepMat(t(c(0,1,0.5,NA)),NMethod,1)
  InitialPars <- RepMat(t(c(NA,NA,NA,1)),NMethod,1)
  ParsBound <- array(NA, dim = c(NMethod,2,4))
  ParsBound[,1,] <- RepMat(t(c(NA,NA,NA,0.01)),NMethod,1)
  ParsBound[,2,] <- RepMat(t(c(NA,NA,NA,20)),NMethod,1)
  NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1)
  ReConstructType <- "separable"
  CVMean <- TRUE
  ModelSelection <- FALSE
  BWMethod <- "df"
  SigmaMethod <- "PACE"
  MatSym <- matrix(c("A1","A2","A3","A4","A2","A5","A6","A7","A3","A6","A4","A8","A4","A7","A8","A9"), 4, 4, byrow = TRUE) # poly method for local smoothing
  InvSym <- InvMat(MatSym = MatSym, Evaluate = FALSE, Output = c("Inverse"), InverseFold = FALSE)
  Inv.Method <- "poly"  
  Phi.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Converge.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Phi.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Converge.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  CompTime <- array(NA, dim = c(B,A,S,M))
  EstimateList <- array(list(), dim = c(B,A,S,M))
  Rho.Ind <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))
  Rho.Proj <- array(NA, dim = c(B,A,S,M,NTarget,max(KAll),NMethod))

  for(b in 1 : B)
  {
    for(m in 1 : M)
    {
      for(s in 1 : S)
      {
        for(a in 1 : A)
        {
          now <- proc.time()[3] 
          thisObs <- SimObservation[[b,a,s,m]]
          thisTime <- SimTime[[b,a,s,m]]
          thisSigma <- SigmaVec[s]
          try(EstimateList[[b,a,s,m]] <- SCorEstMM(Y = thisObs, Time = thisTime, TimeRange = TimeRange, TimeRegular = TimeRegular, EvalTime.Regular = EvalTime.Regular, Rounding = Rounding, 
                                                   EvalTime.Mu = EvalTime.Mu, KAll = KAll, EvalTime.Diag = EvalTime.Diag, Coordinate = Coordinate, Rotate = Rotate, InvSym = InvSym, 
                                                   TargetType = TargetType, TargetAll = TargetAll, TargetIdxList = TargetIdxList, TargetRep = TargetRep, ModelSelection = ModelSelection, 
                                                   DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, Log = Log, Optim.Method = Optim.Method, MaxIt = MaxIt, 
                                                   NGridsSearch = NGridsSearch, Sigma = thisSigma, Dim = Dim, ReConstructType = ReConstructType, CVMean = CVMean, BWMethod = BWMethod, 
                                                   SigmaMethod = SigmaMethod, Inv.Method = Inv.Method), silent = TRUE)
          if(!is.null(EstimateList[[b,a,s,m]]))
          {
            Phi.Ind[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$ParFitted.Ind[,,,4]
            Converge.Ind[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$Converge.Ind 
            Phi.Proj[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$ParFitted.Proj[,,,4]
            Converge.Proj[b,a,s,m,,,] <- EstimateList[[b,a,s,m]]$Converge.Proj
            for(t in 1 : NTarget)
            {
              for(k in 1 : max(KAll))
              {
                for(me in 1 : NMethod)
                {
                  Rho.Ind[b,a,s,m,t,k,me] <- matern(u = 1, kappa = 0.5, phi = Phi.Ind[b,a,s,m,t,k,me])
                  Rho.Proj[b,a,s,m,t,k,me] <- matern(u = 1, kappa = 0.5, phi = Phi.Proj[b,a,s,m,t,k,me])
                }
              }
            }
          }
          CompTime[b,a,s,m] <- proc.time()[3] - now
          print(paste("b =", b, "a =", a, "s =", s, "m =", m, "computing time is", CompTime[b,a,s,m]))

        }
      }
    }
  }

  save.image(paste(corename, ".RData", sep = ""))
  
 