  
  
  
  ############################################################################
  ####   estimate rho for AR1 type spatial correlation on 1-dim locations ####
  ############################################################################
  
  #---------------------------------------------------------------------------------------------------------------------
  # Speicify Directories
  #---------------------------------------------------------------------------------------------------------------------
  
  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v8" # version 8
  
  #---------------------------------------------------------------------------------------------------------------------
  # Load Packages and Functions
  #---------------------------------------------------------------------------------------------------------------------
  
  batch <- 4
  batchsize <- 50
  setwd(workingpath)
  source("UF.SourceDir.r")
  SourceDir(Path = sourcepath, Trace = TRUE)  
  setwd(workingpath)
  corename <- paste("130128.AR1_2Dim_NonSeparable_CorEst_AllTar_Diff_Angle_batch", batch, sep = "")
  library(fda)
  library(sm)
  library(geoR)
  #sink(paste(corename, ".txt", sep = ""))
  
  #---------------------------------------------------------------------------------------------------------------------
  # Generate Simulated Data
  #---------------------------------------------------------------------------------------------------------------------
  
  DistType <- "mahalanobis"
  NEdge <- 10
  NCurve <- NEdge^2
  Coordinate <- cbind(rep(1 : NEdge, each = NEdge), rep(NEdge : 1, times = NEdge))
  Dim <- 2
  AlphaList <- list(c(30,30,30), c(70,45,30))
  DeltaList <- list(c(8,8,8),c(8,8,8))
  KappaList <- list(c(0.5,0.5,0.5),c(0.5,0.5,0.5))
  PhiList <- list(c(5,5,5),c(5,5,5))
  TimeRange <- c(0, 1)
  MPerCurve <- 10
  SigmaVec <- c(1)
  B <- batchsize
  K <- 3
  nbasis <- 2*(floor(K/2)+1)-1
  basis <- create.fourier.basis(TimeRange, nbasis = nbasis)
  MeanFD <- fd(rep(0, nbasis), basis) 
  EigenFunCoefMat <- matrix(0, nbasis, nbasis)
  EigenFunCoefMat[1 : K, 1 : K] <- diag(1, K)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Sign <- as.vector(sign(eval.fd(TimeRange[1] + 0.01, EigenFun)))
  EigenFunCoefMat <- EigenFunCoefMat %*% diag(Sign)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Lambda <- exp(-(1:K)) * 10 
  NGrid <- 101
  Grids <- seq(TimeRange[1], TimeRange[2], length.out = NGrid)
  SimDataList <- SCorDatGen(DistType = DistType, Coordinate = Coordinate, KappaList = KappaList, 
                            PhiList = PhiList, TimeRange = TimeRange, MPerCurve = MPerCurve, SigmaVec = SigmaVec, B = B, MeanFD = MeanFD, 
                            EigenFun = EigenFun, Lambda = Lambda, TimeRegular = TRUE, Grids = Grids, AlphaList = AlphaList, DeltaList = DeltaList)
  SimObservation <- SimDataList$Observation
  SimTime <- SimDataList$Time  
  DistMat <- SimDataList$DistMat[[1]]
  Scores <- SimDataList$Scores

#---------------------------------------------------------------------------------------------------------------------
# Estimate Matern Correlation Parameters
#---------------------------------------------------------------------------------------------------------------------

  A <- length(PhiList)
  M <- length(MPerCurve)
  S <- length(SigmaVec)
  TimeRegular <- TRUE
  EvalTime.Regular <- Grids
  Rounding <- "closest"
  EvalTime.Mu <- EvalTime.Regular
  KAll <- c(3)
  EvalTime.Diag <- seq(0.3,0.7,length.out = 201)
  Rotate <- FALSE
  TargetType <- "cartesian"
  NT <- 24
  BWMethod <- "cv"
  SigmaMethod <- "PACE"
  MatSym <- matrix(c("A1","A2","A3","A4","A2","A5","A6","A7","A3","A6","A4","A8","A4","A7","A8","A9"), 4, 4, byrow = TRUE) # poly method for local smoothing
  InvSym <- InvMat(MatSym = MatSym, Evaluate = FALSE, Output = c("Inverse"), InverseFold = FALSE)
  Inv.Method <- "poly"
  TargetCore <- matrix(c(1,0, 1,1, 0,1, 1,-1, 2,0, 2,1, 2,2, 1,2, 0,2, 1,-2, 2,2, 2,-1, 3,0, 3,1, 3,2, 3,3, 2,3, 1,3, 0,3, 1,-3, 2,-3, 3,3, 3,-2, 3,-1), NT, 2, byrow = TRUE)
  TargetAll <- array(NA, dim = c(NT, 2, 2))
  TargetAll[,1,] <- TargetCore
  TargetAll[,2,] <- TargetCore
  TargetIdxList <- list(1:4,1:6,1:8,1:10,1:12,1:14,1:16,1:18,1:20,1:22,1:24)
  TargetRep <- TargetCore
  Optim.Method <- c("Nelder-Mead", "BFGS")
  NMethod <- length(Optim.Method)
  Log <- RepMat(t(c(FALSE,TRUE,FALSE,FALSE)),NMethod,1)
  MaxIt <- rep(1000,NMethod)
  DefaultPars <- RepMat(t(c(NA,NA,0.5,NA)),NMethod,1)
  InitialPars <- RepMat(t(c(0,0,NA,0.5)),NMethod,1)
  ParsBound <- array(NA, dim = c(NMethod,2,4))
  ParsBound[,1,] <- RepMat(t(c(-360,-10,NA,0.01)),NMethod,1)
  ParsBound[,2,] <- RepMat(t(c(360,10,NA,50)),NMethod,1)
  NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1)
  ReConstructType <- "separable"
  CVMean <- TRUE
  ModelSelection <- FALSE
  InvSym <- NULL
  CompTime <- array(NA, dim = c(B,A,S,M))
  EstimateList <- array(list(), dim = c(B,A,S,M))
 
  for(b in 1 : B)
  {
    for(m in 1 : M)
    {
      for(s in 1 : S)
      {
        for(a in 1 : A)
        {
          now <- proc.time()[3] 
          thisObs <- SimObservation[[b,a,s,m]]
          thisTime <- SimTime[[b,a,s,m]]
          thisSigma <- SigmaVec[s]
          try(EstimateList[[b,a,s,m]] <- SCorEstMM(Y = thisObs, Time = thisTime, TimeRange = TimeRange, TimeRegular = TimeRegular, EvalTime.Regular = EvalTime.Regular, Rounding = Rounding, 
                                                   EvalTime.Mu = EvalTime.Mu, KAll = KAll, EvalTime.Diag = EvalTime.Diag, Coordinate = Coordinate, Rotate = Rotate, InvSym = InvSym, 
                                                   TargetType = TargetType, TargetAll = TargetAll, TargetIdxList = TargetIdxList, TargetRep = TargetRep, ModelSelection = ModelSelection, 
                                                   DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, Log = Log, Optim.Method = Optim.Method, MaxIt = MaxIt, 
                                                   NGridsSearch = NGridsSearch, Sigma = thisSigma, Dim = Dim, ReConstructType = ReConstructType, CVMean = CVMean, BWMethod = BWMethod, 
                                                   SigmaMethod = SigmaMethod, Inv.Method = Inv.Method), silent = TRUE) 
          CompTime[b,a,s,m] <- proc.time()[3] - now
          print(paste("b =", b, "a =", a, "s =", s, "m =", m, "computing time is", CompTime[b,a,s,m]))

        }
      }
    }
  }

save.image(paste(corename, ".RData", sep = ""))
  
  #sink()

 