  
  #---------------------------------------------------------------------------------------------------------------------
  # Speicify Directories
  #---------------------------------------------------------------------------------------------------------------------
  
  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v11"
  HarvardDatapath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/PPC_v1"
  
  #---------------------------------------------------------------------------------------------------------------------
  # Load Packages and Functions
  #---------------------------------------------------------------------------------------------------------------------
  
    setwd(workingpath)
    source("UF.SourceDir.r")
    SourceDir(Path = sourcepath, Trace = TRUE)  
    setwd(workingpath)
    corename <- "130804.AR1_2Dim_Harvard_Forest_Separability_Test"
    library(fda)
    library(sm)
    library(Ryacas)
    library(geoR)
  
  #---------------------------------------------------------------------------------------------------------------------
  # Load Raw Harvard Forest Data
  #---------------------------------------------------------------------------------------------------------------------
  
    setwd(HarvardDatapath)
    load("Harvard_VI.RData")
    setwd(workingpath)
  
  #---------------------------------------------------------------------------------------------------------------------
  # Generate Harvard Forest Data with Manual Missing Data
  #---------------------------------------------------------------------------------------------------------------------
 
    Coordinate <- cbind(rep(1 : 25, each = 25), rep(25 : 1, times = 25))
    YearIndex <- rep(1 : 6, each = 46)
    YearOfInterest <- 5
    PixelOfInterest <- which(Coordinate[,1] >= 8 & Coordinate[,2] >= 8 & Coordinate[,1] <= 18 & Coordinate[,2] <= 18)
    EVISmall <- EVIfull[PixelOfInterest, YearIndex == YearOfInterest]
    EVISmall[EVISmall < threshold] <- NA
    matplot(t(EVISmall))
    RawMissing <- apply(EVISmall, 1, function(x){which(is.na(x))})
    RawNonMissing <- apply(EVISmall, 1, function(x){which(!is.na(x))})
    RawMissingNum <- sapply(RawMissing, length)
    RawMissingCurveID <- which(RawMissingNum > 0)    
    ObsPerCurve <- 10
    B <- 20
    NumCurves <- dim(EVISmall)[1]
    NumTimePts <- dim(EVISmall)[2]
    Times <- list()
    Observations <- list() 
    ObsIndexes <- list()
    Grids <- ((1 : NumTimePts) - 0.5) / NumTimePts
  
    for(b in 1 : B)
    {
      Observations[[b]] <- list()
      Times[[b]] <- list()
      ObsIndexes[[b]] <- list()
      for(c in 1 : NumCurves)
      {
        if(NumTimePts - RawMissingNum[c] < ObsPerCurve)
        {
          ObsIndexes[[b]][[c]] <- RawNonMissing[[c]]  
        }else
        {
          ObsIndexes[[b]][[c]] <- sort(sample(RawNonMissing[[c]], ObsPerCurve, replace = FALSE))
        }
        Observations[[b]][[c]] <- EVISmall[c, ObsIndexes[[b]][[c]]]
        Times[[b]][[c]] <- Grids[ObsIndexes[[b]][[c]]] 
      }
      print(paste("Semi empirical data set b = ", b, " is generated", sep = ""))
    }
  
    plot(Times[[1]][[1]], Observations[[1]][[1]], pch = 19, ylim = c(0, 0.8), xlim = c(0, 1))  
    for(c in 2 : NumCurves)
    {
      points(Times[[1]][[c]], Observations[[1]][[c]], pch = 19, col = c) 
    }
  
  #---------------------------------------------------------------------------------------------------------------------
  # Fit Simulated Data
  #---------------------------------------------------------------------------------------------------------------------

    SCorEstMMParams <- list()
    SCorEstMMParams$TimeRange <- c(0,1)
    SCorEstMMParams$TimeRegular <- TRUE
    SCorEstMMParams$EvalTime.Regular <- Grids
    SCorEstMMParams$Rounding <- "closest"
    SCorEstMMParams$EvalTime.Mu <- SCorEstMMParams$EvalTime.Regular
    SCorEstMMParams$KAll <- 7
    SCorEstMMParams$EvalTime.Diag <- seq(0.3,0.7,length.out = 201)
    MatSym <- matrix(c("A1","A2","A3","A4","A2","A5","A6","A7","A3","A6","A4","A8","A4","A7","A8","A9"), 4, 4, byrow = TRUE) # poly method for local smoothing
    SCorEstMMParams$LLQParams <- list()
    SCorEstMMParams$LLQParams$Method <- "poly"
    SCorEstMMParams$LLQParams$InvSym <- InvMat(MatSym = MatSym, Evaluate = FALSE, Output = c("Inverse"), InverseFold = FALSE)
    SCorEstMMParams$Coordinate <- cbind(rep(1:sqrt(NumCurves), each = sqrt(NumCurves)), rep(sqrt(NumCurves):1, times = sqrt(NumCurves)))
    SCorEstMMParams$Rotate <- FALSE
    SCorEstMMParams$TargetType <- "cartesian"
    NT <- 24
    TargetCore <- matrix(c(1,0, 1,1, 0,1, 1,-1, 2,0, 2,1, 2,2, 1,2, 0,2, 1,-2, 2,2, 2,-1, 3,0, 3,1, 3,2, 3,3, 2,3, 1,3, 0,3, 1,-3, 2,-3, 3,3, 3,-2, 3,-1), NT, 2, byrow = TRUE)
    SCorEstMMParams$TargetAll <- array(NA, dim = c(NT, 2, 2))
    SCorEstMMParams$TargetAll[,1,] <- TargetCore
    SCorEstMMParams$TargetAll[,2,] <- TargetCore
    SCorEstMMParams$TargetIdxList <- list(1:24)
    SCorEstMMParams$TargetRep <- TargetCore
    SCorEstMMParams$Optim.Method <- c("BFGS")
    NMethod <- length(SCorEstMMParams$Optim.Method)
    SCorEstMMParams$Log <- RepMat(t(c(FALSE,TRUE,FALSE,FALSE)), NMethod,1)
    SCorEstMMParams$MaxIt <- rep(1000,NMethod)
    SCorEstMMParams$DefaultPars <- RepMat(t(c(0,0,0.5,NA)),NMethod,1)
    SCorEstMMParams$InitialPars <- RepMat(t(c(NA,NA,NA,0.5)),NMethod,1)
    SCorEstMMParams$ParsBound <- array(NA, dim = c(NMethod,2,4))
    SCorEstMMParams$ParsBound[,1,] <- RepMat(t(c(-360,-10,NA,0.01)),NMethod,1)
    SCorEstMMParams$ParsBound[,2,] <- RepMat(t(c(360,10,NA,50)),NMethod,1)
    SCorEstMMParams$NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1)
    SCorEstMMParams$ReConstructType <- "separable"  
    SCorEstMMParams$BWMethod <- "cv"
    SCorEstMMParams$SigmaMethod <- "PACE"
    SCorEstMMParams$CVMean <- TRUE
    SCorEstMMParams$D1NFold <- NA
    SCorEstMMParams$D1Separation <- NA
    SCorEstMMParams$ModelSelection <- FALSE
    SCorEstMMParams$Verbose <- FALSE
    SCorEstMMParams$Dim <- 2
    SCorEstMMParams$Sigma <- 0.1
    SCorEstMMParams$DF <- NA
    SCorEstMMParams$ResModelK <- SCorEstMMParams$KAll
    SCorEstMMParams$Reconstruction <- FALSE
  
    CompTime <- c()
    EstimateList <- list()
    TestResult <- list()
  
    NResample <- 100
    KAll <- SCorEstMMParams$KAll
  
    for(b in 1 : 1)
    {
      now <- proc.time()[3]
      thisObs <- Observations[[b]]
      thisTime <- Times[[b]]
      thisEstimateList <- NULL
      try(thisEstimateList <- SCorEstMM(Y = thisObs, Time = thisTime, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      CompTime[b] <- proc.time()[3] - now
      print(paste("b =", b, "simulated curves estimation is done using", CompTime[b], 'seconds'))
      if(is.null(thisEstimateList))
      {
        print(paste("b =", b, "simulated curves estimation is bad !!!!!!!!!!!!!"))
        next
      }else 
      {
        EstimateList[[b]] <- thisEstimateList
        PhiEst.Full <- apply(thisEstimateList$ParFitted.Ind[,,1,4], 2, mean) # phi estimation using full model with no restriction
        PhiEst.Restrict <- mean(thisEstimateList$ParFittedPull.Ind[,1,4]) # phi estimation using pooled correlations        
        Par.Full <- cbind(RepMat(cbind(0,1,0.5),SCorEstMMParams$KAll,1), PhiEst.Full)
        Par.Restrict <- cbind(0,1,0.5,PhiEst.Restrict)
        TestResult[[b]] <- SeparabilityTest(Par.Full = Par.Full, Par.Restrict = Par.Restrict, Coordinate = SCorEstMMParams$Coordinate, K = SCorEstMMParams$KAll, EstimateList = thisEstimateList, NResample = NResample, SCorEstMMParams = SCorEstMMParams, Time = thisTime)
        CompTime[b] <- proc.time()[3] - now      
        print(paste("b =", b, "simulated curve test is done using", CompTime[b], 'seconds'))
      }      
    }
 
  #---------------------------------------------------------------------------------------------------------------------
  # Summarize Separability Test Results
  #---------------------------------------------------------------------------------------------------------------------

    ObservePhiSD <- c()
    ObserveCorSD <- c()
    PermPhi <- array(NA, dim = c(B, NResample, KAll))
    PermCor <- array(NA, dim = c(B, NResample, KAll))
    PermPhiSD <- matrix(NA, B, NResample)
    PermCorSD <- matrix(NA, B, NResample)
    PhiEst.Full <- matrix(NA, B, KAll)
    CorEst.Full <- matrix(NA, B, KAll)
    Index <- 1 : 7
  
    for(b in 1 : 1)
    {
      PhiEst.Full[b,] <- apply(EstimateList[[b]]$ParFitted.Ind[,,1,4], 2, median) # phi estimation using full model with no restriction
      ObservePhiSD[b] <- sd(PhiEst.Full[b,Index], na.rm = TRUE) 
      PermPhi[b,,] <- apply(TestResult[[b]]$PermPhiEst.UnRotate, c(1,3), median)
      PermPhiSD[b,] <- apply(PermPhi[b,,Index], 1, sd, na.rm = TRUE) 
      
      for(k in 1 : KAll)
      {
        CorEst.Full[b,k] <- matern(u = 1, kappa = 0.5, phi = PhiEst.Full[b,k])
        ObserveCorSD[b] <- sd(CorEst.Full[b,Index], na.rm = TRUE) 
        for(p in 1 : NResample)
        {
          PermCor[b,p,k] <- matern(u = 1, kappa = 0.5, phi = PermPhi[b,p,k])
        }
      }
      PermCorSD[b,] <- apply(PermCor[b,,Index], 1, sd, na.rm = TRUE)
    }
    
  save.image(paste(corename, ".RData", sep = ""))
  
  #---------------------------------------------------------------------------------------------------------------------
  # Plot Reconstruction Results
  #---------------------------------------------------------------------------------------------------------------------
 
    pdf(paste(corename, ".pdf", sep = ""), width = 14, height = 9)
    par(mfcol = c(2,3))
    for(b in 1 : 1)
    {
      hist(log(PermPhiSD[b,]), xlim = c(-10, 10), main = "")
      abline(v = log(ObservePhiSD[b]), col = 2, lwd = 3)
      title("log Phi standard deviation of", line = 3)
      title(paste("the first", max(Index), "eigenvalues"), line = 2)
    }
    for(b in 1 : 1)
    {
      hist(PermCorSD[b,], xlim = c(-0.2, 1), main = "")
      abline(v = ObserveCorSD[b], col = 2, lwd = 3)
      title("Cor standard deviation of", line = 3)
      title(paste("the first", max(Index), "eigenvalues"), line = 2)      
    }  
    dev.off()
  
  