  
  #---------------------------------------------------------------------------------------------------------------------
  # Speicify Directories
  #---------------------------------------------------------------------------------------------------------------------
  
  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v11" # version 11
  
  #---------------------------------------------------------------------------------------------------------------------
  # Load Packages and Functions
  #---------------------------------------------------------------------------------------------------------------------
  
  setwd(workingpath)
  source("UF.SourceDir.r")
  SourceDir(Path = sourcepath, Trace = TRUE)  
  setwd(workingpath)
  Batch <- 1
  BatchSize <- 25
  TransformType <- 1
  corename <- paste("130806.AR1_1Dim_Separable_H0_Diff_Sigma_More_Lags_Low_Cor_Batch_", Batch, "_Transform_", TransformType, sep = "")
  library(fda)
  library(sm)
  library(Ryacas)
  library(geoR)
  library(abind)
  
  #---------------------------------------------------------------------------------------------------------------------
  # Generate Simulated Data
  #---------------------------------------------------------------------------------------------------------------------
  
  DistType <- "euclidean"
  NCurve <- 100
  Coordinate <- cbind(1 : NCurve, 0)
  Dim <- 1
  AlphaList <- list(c(0,0))
  DeltaList <- list(c(1,1))
  KappaList <- list(c(0.5,0.5))
  PhiList <- list(c(1,1))
  TimeRange <- c(0,1)
  MPerCurve <- 10
  SigmaVec <- c(1)
  B <- BatchSize
  K <- 2
  basis <- create.fourier.basis(TimeRange, nbasis = 3)
  MeanFD <- fd(rep(0, 3), basis)  
  EigenFunCoefMat <- matrix(0, 3, K)
  EigenFunCoefMat[1:K, 1:K] <- diag(1, K)
  EigenFun <- fd(EigenFunCoefMat, basis)
  Lambda <- exp(-(1:K)) * 10 
  NGrid <- 101
  Grids <- seq(TimeRange[1], TimeRange[2], length.out = NGrid)
  StepSize <- (TimeRange[2] - TimeRange[1]) / (NGrid - 1) 
  SimDataList <- SCorDatGen(DistType = DistType, Coordinate = Coordinate, KappaList = KappaList, PhiList = PhiList, TimeRange = TimeRange, 
                            MPerCurve = MPerCurve, SigmaVec = SigmaVec, B = B, MeanFD = MeanFD, EigenFun = EigenFun, Lambda = Lambda, TimeRegular = TRUE, 
                            Grids = Grids, AlphaList = AlphaList, DeltaList = DeltaList)
  SimObservation <- SimDataList$Observation
  SimTime <- SimDataList$Time  
  DistMat <- SimDataList$DistMat[[1]]
  
  #---------------------------------------------------------------------------------------------------------------------
  # Estimate Matern Correlation Parameters
  #---------------------------------------------------------------------------------------------------------------------
  
  A <- length(PhiList)
  M <- length(MPerCurve)
  S <- length(SigmaVec)
  SCorEstMMParams <- list()
  SCorEstMMParams$TimeRange <- c(0,1)
  SCorEstMMParams$TimeRegular <- TRUE
  SCorEstMMParams$EvalTime.Regular <- Grids
  SCorEstMMParams$Rounding <- "closest"
  SCorEstMMParams$EvalTime.Mu <- SCorEstMMParams$EvalTime.Regular
  SCorEstMMParams$KAll <- 2
  SCorEstMMParams$EvalTime.Diag <-seq(0.3,0.7,length.out = 201)
  MatSym <- matrix(c("A1","A2","A3","A4","A2","A5","A6","A7","A3","A6","A4","A8","A4","A7","A8","A9"), 4, 4, byrow = TRUE) # poly method for local smoothing
  InvSym <- InvMat(MatSym = MatSym, Evaluate = FALSE, Output = c("Inverse"), InverseFold = FALSE)
  SCorEstMMParams$LLQParams <- list()
  SCorEstMMParams$LLQParams$Method <- "poly"
  SCorEstMMParams$LLQParams$InvSym <- InvSym
  SCorEstMMParams$Coordinate <- cbind(1 : NCurve, 0)
  SCorEstMMParams$Rotate <- FALSE  
  SCorEstMMParams$TargetType <- "polar"
  NTarget <- 2
  TargetCore <- 1 : NTarget
  SCorEstMMParams$TargetAll <- array(NA, dim = c(NTarget,2,2))
  SCorEstMMParams$TargetAll[,,1] <- cbind(TargetCore, TargetCore)
  SCorEstMMParams$TargetAll[,,2] <- cbind(rep(-91, NTarget), rep(91, NTarget))
  SCorEstMMParams$TargetIdxList <- list(1)
  SCorEstMMParams$ModelSelection <- FALSE
  SCorEstMMParams$TargetRep <- cbind(TargetCore,0)
  SCorEstMMParams$Optim.Method <- "BFGS"
  NMethod <- length(SCorEstMMParams$Optim.Method)
  SCorEstMMParams$DefaultPars <- RepMat(t(c(0,1,0.5,NA)),NMethod,1)
  SCorEstMMParams$InitialPars <- RepMat(t(c(NA,NA,NA,0)),NMethod,1)
  SCorEstMMParams$ParsBound <- array(NA, dim = c(NMethod,2,4))
  SCorEstMMParams$ParsBound[,1,] <- RepMat(t(c(NA,NA,NA,0.01)),NMethod,1)
  SCorEstMMParams$ParsBound[,2,] <- RepMat(t(c(NA,NA,NA,20)),NMethod,1)
  SCorEstMMParams$Log <- RepMat(t(c(FALSE,FALSE,FALSE,TRUE)),NMethod,1)   
  SCorEstMMParams$NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1) 
  SCorEstMMParams$MaxIt <- rep(200, NMethod)      
  SCorEstMMParams$Dim <- 1
  SCorEstMMParams$ReConstructType <- "separate"
  SCorEstMMParams$Sigma <- NA
  SCorEstMMParams$CVMean <- TRUE
  SCorEstMMParams$D1NFold <- 5
  SCorEstMMParams$D1Separation <- 10
  SCorEstMMParams$SigmaMethod <- "PACE"
  SCorEstMMParams$Verbose <- TRUE
  SCorEstMMParams$BWMethod <- "cv"
  SCorEstMMParams$DF <- NA
  SCorEstMMParams$ResModelK <- 2  
  SCorEstMMParams$Reconstruction <- FALSE
  NResample <- 100  
  CompTime <- array(NA, dim = c(B,A,S,M)) 
  TestResult <- array(list(), dim = c(B,A,S,M))
  EstimateList <- array(list(), dim = c(B,A,S,M))
  
  for(b in 12 : B)
  {
    for(m in 1 : M)
    {
      for(s in 1 : S)
      {
        for(a in 1 : A)
        { 
          now <- proc.time()[3]
          thisObs <- SimObservation[[b,a,s,m]]
          thisTime <- SimTime[[b,a,s,m]]
          thisEstimateList <- NULL
          try(thisEstimateList <- SCorEstMM(Y = thisObs, Time = thisTime, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
          CompTime[b] <- proc.time()[3] - now
          print(paste("b =", b, "simulated curves estimation is done using", CompTime[b], 'seconds'))
          if(is.null(thisEstimateList))
          {
            print(paste("b =", b, "simulated curves estimation is bad !!!!!!!!!!!!!"))
            next
          }else 
          {
            EstimateList[[b,a,s,m]] <- thisEstimateList
            PhiEst.Full <- apply(thisEstimateList$ParFitted.Ind[,,1,4], 2, mean) # phi estimation using full model with no restriction
            PhiEst.Restrict <- mean(thisEstimateList$ParFittedPull.Ind[,1,4]) # phi estimation using pooled correlations        
            Par.Full <- cbind(RepMat(cbind(0,1,0.5),SCorEstMMParams$KAll,1), PhiEst.Full)
            Par.Restrict <- cbind(0,1,0.5,PhiEst.Restrict)
            TestResult[[b,a,s,m]] <- SeparabilityTest(Par.Full = Par.Full, Par.Restrict = Par.Restrict, Coordinate = SCorEstMMParams$Coordinate, K = SCorEstMMParams$KAll, EstimateList = thisEstimateList, NResample = NResample, SCorEstMMParams = SCorEstMMParams, Time = thisTime, TransformType = TransformType)
            CompTime[b] <- proc.time()[3] - now      
            print(paste("b =", b, "simulated curve test is done using", CompTime[b], 'seconds'))
          }
        }
      } 
    }
  }
  
  PhiEst.Full <- array(NA, dim = c(B,A,S,M,K))
  PermPhiEst.Full.UnRotate <- array(NA, dim = c(B,A,S,M,NResample,K))
  BootPhiEst.Full.UnRotate <- array(NA, dim = c(B,A,S,M,NResample,K))
  
  for(b in 1 : B)
  {
    for(m in 1 : M)
    {
      for(s in 1 : S)
      {
        for(a in 1 : A)
        { 
          thisEstimateList <- EstimateList[[b,a,s,m]] 
          thisTestResult <- TestResult[[b,a,s,m]]
          if(!is.null(thisTestResult) & !is.null(thisEstimateList))
          {  
            PhiEst.Full[b,a,s,m,] <- apply(thisEstimateList$ParFitted.Ind[,,1,4], 2, mean) # phi estimation using full model with no restriction
            PermPhiEst.Full.UnRotate[b,a,s,m,,] <- apply(thisTestResult$PermPhiEst.UnRotate, c(1,3), mean)
            BootPhiEst.Full.UnRotate[b,a,s,m,,] <- apply(thisTestResult$BootPhiEst.UnRotate, c(1,3), mean)          
          }
        }
      } 
    }
  }  
  
  PhiDiff <- PhiEst.Full[,,,,1] - PhiEst.Full[,,,,2] # difference between Phi of different eigenvalues under full model
  PhiAbsDiff <- abs(PhiDiff) # absolute Phi difference 
  PermPhiAbsDiff.UnRotate <- abs(PermPhiEst.Full.UnRotate[,,,,,1] - PermPhiEst.Full.UnRotate[,,,,,2]) # absolute perm phi difference
  BootPhiAbsDiff.UnRotate <- abs(BootPhiEst.Full.UnRotate[,,,,,1] - BootPhiEst.Full.UnRotate[,,,,,2]) # absolute boot phi difference
  PermPhiAbsDiffCritical.UnRotate <- apply(PermPhiAbsDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE)
  PermPhiRej.UnRotate <- PhiAbsDiff > PermPhiAbsDiffCritical.UnRotate
  BootPhiAbsDiffCritical.UnRotate <- apply(BootPhiAbsDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE)
  BootPhiRej.UnRotate <- PhiAbsDiff > BootPhiAbsDiffCritical.UnRotate  
  PermCorEst.Full.UnRotate <-apply(PermPhiEst.Full.UnRotate, MARGIN = 1:6, FUN = matern, u = 1, kappa = 0.5)
  BootCorEst.Full.UnRotate <-apply(BootPhiEst.Full.UnRotate, MARGIN = 1:6, FUN = matern, u = 1, kappa = 0.5)  
  PermCorDiff.UnRotate <- PermCorEst.Full.UnRotate[,,,,,1] - PermCorEst.Full.UnRotate[,,,,,2]
  BootCorDiff.UnRotate <- BootCorEst.Full.UnRotate[,,,,,1] - BootCorEst.Full.UnRotate[,,,,,2] 
  PermCorAbsDiff.UnRotate <- abs(PermCorDiff.UnRotate)
  BootCorAbsDiff.UnRotate <- abs(BootCorDiff.UnRotate)  
  CorEst.Full <- apply(PhiEst.Full, MARGIN = 1:5, FUN = matern, u = 1, kappa = 0.5)
  CorDiff <- CorEst.Full[,,,,1] - CorEst.Full[,,,,2]
  CorAbsDiff <- abs(CorDiff)
  PermCorAbsDiffCritical.UnRotate <- apply(PermCorAbsDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE)
  PermCorAbsRej.UnRotate <- CorAbsDiff > PermCorAbsDiffCritical.UnRotate
  BootCorAbsDiffCritical.UnRotate <- apply(BootCorAbsDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE)
  BootCorAbsRej.UnRotate <- CorAbsDiff > BootCorAbsDiffCritical.UnRotate
  PermCorDiffCriticalHigh.UnRotate <- apply(PermCorDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.975, na.rm = TRUE)
  PermCorDiffCriticalLow.UnRotate <- apply(PermCorDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.025, na.rm = TRUE)
  PermCorRej.UnRotate <- CorDiff > PermCorDiffCriticalHigh.UnRotate | CorDiff < PermCorDiffCriticalLow.UnRotate
  BootCorDiffCriticalHigh.UnRotate <- apply(BootCorDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.975, na.rm = TRUE)
  BootCorDiffCriticalLow.UnRotate <- apply(BootCorDiff.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.025, na.rm = TRUE)
  BootCorRej.UnRotate <- CorDiff > BootCorDiffCriticalHigh.UnRotate | CorDiff < BootCorDiffCriticalLow.UnRotate
  PermPhiStd.UnRotate <- drop(apply(PermPhiEst.Full.UnRotate, 1:5, sd))
  BootPhiStd.UnRotate <- drop(apply(BootPhiEst.Full.UnRotate, 1:5, sd))
  PhiStd <- drop(apply(PhiEst.Full, 1:4, sd))
  PermPhiStdCritical.UnRotate <- drop(apply(PermPhiStd.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE))
  BootPhiStdCritical.UnRotate <- drop(apply(BootPhiStd.UnRotate, MARGIN = 1, FUN = quantile, prob = 0.95, na.rm = TRUE))
  PermStdRej <- PhiStd > PermPhiStdCritical.UnRotate 
  BootStdReg <- PhiStd > BootPhiStdCritical.UnRotate
  
  save.image(paste(corename, ".RData", sep = ""))

  
  # Absolute difference of Phi
  pdf(paste(corename,".pdf", sep = ""), width = 14, height = 9)

  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    {    
      hist(log(PermPhiAbsDiff.UnRotate[i,]), xlim = c(-10, 10), main = "", xlab = "log abs diff of phi")
      abline(v = log(PhiAbsDiff[i]), lwd = 3, col = 2)
      abline(v = log(PermPhiAbsDiffCritical.UnRotate[i]), lwd = 3, col = 3)
      title("permutation unrotate", line = 2)
      title(paste("log abs diff of phi, curve", i), line = 1)
    }
  }   

  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    {   
      hist(log(BootPhiAbsDiff.UnRotate[i,]), xlim = c(-10, 10), main = "", xlab = "log abs diff of phi")
      abline(v = log(PhiAbsDiff[i]), lwd = 3, col = 2)
      abline(v = log(BootPhiAbsDiffCritical.UnRotate[i]), lwd = 3, col = 3)
      title("bootstrap unrotate", line = 2)
      title(paste("log abs diff of phi, curve", i), line = 1)
    }
  } 
  
  # Absolute difference of Correlation

  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(PermCorAbsDiff.UnRotate[i,], xlim = c(0, 1), main = "", xlab = "abs diff of Cor")
      abline(v = CorAbsDiff[i], lwd = 3, col = 2)
      abline(v = PermCorAbsDiffCritical.UnRotate[i], lwd = 3, col = 3)
      title("permutation unrotate", line = 2)
      title(paste("abs diff of Cor, curve", i), line = 1)
    }
  }   
  
  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(BootCorAbsDiff.UnRotate[i,], xlim = c(0, 1), main = "", xlab = "abs diff of Cor")
      abline(v = CorAbsDiff[i], lwd = 3, col = 2)
      abline(v = BootCorAbsDiffCritical.UnRotate[i], lwd = 3, col = 3)
      title("bootstrap unrotate", line = 2)
      title(paste("abs diff of Cor, curve", i), line = 1)
    }
  }   
  
  # Difference of Correlation
  
  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(PermCorDiff.UnRotate[i,], xlim = c(-1, 1), main = "", xlab = "diff of Cor")
      abline(v = CorDiff[i], lwd = 3, col = 2)
      abline(v = PermCorDiffCriticalHigh.UnRotate[i], lwd = 3, col = 3)
      abline(v = PermCorDiffCriticalLow.UnRotate[i], lwd = 3, col = 3)
      title("permutation unrotate", line = 2)
      title(paste("diff of Cor, curve", i), line = 1)
    }
  }   

  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(BootCorDiff.UnRotate[i,], xlim = c(-1, 1), main = "", xlab = "diff of Cor")
      abline(v = CorDiff[i], lwd = 3, col = 2)
      abline(v = BootCorDiffCriticalHigh.UnRotate[i], lwd = 3, col = 3)
      abline(v = BootCorDiffCriticalLow.UnRotate[i], lwd = 3, col = 3)
      title("bootstrap unrotate", line = 2)
      title(paste("diff of Cor, curve", i), line = 1)
    }
  }  

  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(PermPhiStd.UnRotate[i,], main = "", xlab = "diff of Cor")
      abline(v = PhiStd[i], lwd = 3, col = 2)
      abline(v = PermPhiStdCritical.UnRotate[i], lwd = 3, col = 3)
      title("permutation unrotate", line = 2)
      title(paste("std, curve", i), line = 1)
    }
  }    
  
  par(mfrow = c(5,5))
  for(i in 1 : B)
  {
    if(i == 11)
    {
      hist(0)
    }else
    { 
      hist(BootPhiStd.UnRotate[i,], main = "", xlab = "diff of Cor")
      abline(v = PhiStd[i], lwd = 3, col = 2)
      abline(v = BootPhiStdCritical.UnRotate[i], lwd = 3, col = 3)
      title("bootstrap unrotate", line = 2)
      title(paste("std, curve", i), line = 1)
    }
  }    
  
  dev.off()
  
  