  
  #---------------------------------------------------------------------------------------------------------------------
  # Speicify Directories
  #---------------------------------------------------------------------------------------------------------------------
  
  rm(list = ls())
  workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
  graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
  sourcepath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/Dissertation/SCFDA_v12"
  
  #---------------------------------------------------------------------------------------------------------------------
  # Load Packages and Functions
  #---------------------------------------------------------------------------------------------------------------------
  
    setwd(workingpath)
    source("UF.SourceDir.r")
    SourceDir(Path = sourcepath, Trace = TRUE)  
    setwd(workingpath)
    corename <- "130822.AR1_2Dim_Isotropy_Test"
    library(fda)
    library(sm)
    library(Ryacas)
    library(geoR)
  
  #---------------------------------------------------------------------------------------------------------------------
  #
  #---------------------------------------------------------------------------------------------------------------------
  
    DistType <- "mahalanobis"
    NEdge <- 10
    NCurve <- NEdge^2
    Coordinate <- cbind(rep(1 : NEdge, each = NEdge), rep(NEdge : 1, times = NEdge))
    Dim <- 2
    AlphaList <- list(c(30,30))
    DeltaList <- list(c(8,8))
    KappaList <- list(c(0.5,0.5))
    PhiList <- list(c(5,5))
    TimeRange <- c(0, 1)
    MPerCurve <- 10
    SigmaVec <- c(1)
    B <- 25
    K <- 2
    nbasis <- 2*(floor(K/2)+1)-1
    basis <- create.fourier.basis(TimeRange, nbasis = nbasis)
    MeanFD <- fd(rep(0, nbasis), basis) 
    EigenFunCoefMat <- matrix(0, 3, K)
    EigenFunCoefMat[1:K, 1:K] <- diag(1, K)
    EigenFun <- fd(EigenFunCoefMat, basis)
    Lambda <- exp(-(1:K)) * 10 
    NGrid <- 101
    Grids <- seq(TimeRange[1], TimeRange[2], length.out = NGrid)
    SimDataList <- SCorDatGen(DistType = DistType, Coordinate = Coordinate, KappaList = KappaList, PhiList = PhiList, TimeRange = TimeRange, 
                              MPerCurve = MPerCurve, SigmaVec = SigmaVec, B = B, MeanFD = MeanFD, EigenFun = EigenFun, Lambda = Lambda, TimeRegular = TRUE, 
                              Grids = Grids, AlphaList = AlphaList, DeltaList = DeltaList)
    Observations <- SimDataList$Observation
    Times <- SimDataList$Time  
    DistMat <- SimDataList$DistMat[[1]]
    Scores <- SimDataList$Scores

  #---------------------------------------------------------------------------------------------------------------------
  # Fit Simulated Data
  #---------------------------------------------------------------------------------------------------------------------

    SCorEstMMParams <- list()
    SCorEstMMParams$TimeRange <- c(0,1)
    SCorEstMMParams$TimeRegular <- TRUE
    SCorEstMMParams$EvalTime.Regular <- Grids
    SCorEstMMParams$Rounding <- "closest"
    SCorEstMMParams$EvalTime.Mu <- SCorEstMMParams$EvalTime.Regular
    SCorEstMMParams$KAll <- 2
    SCorEstMMParams$EvalTime.Diag <- seq(0.3,0.7,length.out = 201)
    MatSym <- matrix(c("A1","A2","A3","A4","A2","A5","A6","A7","A3","A6","A4","A8","A4","A7","A8","A9"), 4, 4, byrow = TRUE) # poly method for local smoothing
    SCorEstMMParams$LLQParams <- list()
    SCorEstMMParams$LLQParams$Method <- "poly"
    SCorEstMMParams$LLQParams$InvSym <- InvMat(MatSym = MatSym, Evaluate = FALSE, Output = c("Inverse"), InverseFold = FALSE)
    SCorEstMMParams$Coordinate <- Coordinate
    SCorEstMMParams$Rotate <- TRUE
    SCorEstMMParams$TargetType <- "cartesian"
    NT <- 24
    TargetCore <- matrix(c(1,0, 1,1, 0,1, 1,-1, 2,0, 2,1, 2,2, 1,2, 0,2, 1,-2, 2,-2, 2,-1, 3,0, 3,1, 3,2, 3,3, 2,3, 1,3, 0,3, 1,-3, 2,-3, 3,-3, 3,-2, 3,-1), NT, 2, byrow = TRUE)
    #TargetCore <- matrix(c(1,0, 1,1), NT, 2, byrow = TRUE)
    SCorEstMMParams$TargetAll <- array(NA, dim = c(NT, 2, 2))
    SCorEstMMParams$TargetAll[,1,] <- TargetCore
    SCorEstMMParams$TargetAll[,2,] <- TargetCore
    SCorEstMMParams$TargetIdxList <- list(1:24)
    SCorEstMMParams$TargetRep <- TargetCore
    SCorEstMMParams$Optim.Method <- c("BFGS")
    NMethod <- length(SCorEstMMParams$Optim.Method)
    SCorEstMMParams$Log <- RepMat(t(c(FALSE,TRUE,FALSE,FALSE)), NMethod,1)
    SCorEstMMParams$MaxIt <- rep(1000,NMethod)
    SCorEstMMParams$DefaultPars <- RepMat(t(c(NA,NA,0.5,5)),NMethod,1)
    SCorEstMMParams$InitialPars <- RepMat(t(c(0,0,NA,NA)),NMethod,1)
    SCorEstMMParams$ParsBound <- array(NA, dim = c(NMethod,2,4))
    SCorEstMMParams$ParsBound[,1,] <- RepMat(t(c(-360,-10,NA,0.01)),NMethod,1)
    SCorEstMMParams$ParsBound[,2,] <- RepMat(t(c(360,10,NA,50)),NMethod,1)
    SCorEstMMParams$NGridsSearch <- RepMat(t(c(NA,NA,NA,NA)),NMethod,1)
    SCorEstMMParams$ReConstructType <- "separable"  
    SCorEstMMParams$BWMethod <- "cv"
    SCorEstMMParams$SigmaMethod <- "PACE"
    SCorEstMMParams$CVMean <- TRUE
    SCorEstMMParams$D1NFold <- NA
    SCorEstMMParams$D1Separation <- NA
    SCorEstMMParams$ModelSelection <- FALSE
    SCorEstMMParams$Verbose <- TRUE
    SCorEstMMParams$Dim <- 2
    SCorEstMMParams$Sigma <- 1
    SCorEstMMParams$DF <- NA
    SCorEstMMParams$ResModelK <- SCorEstMMParams$KAll
    SCorEstMMParams$Reconstruction <- FALSE
  
    CompTime <- c()
    EstimateList <- list()
    TestResult <- list()  
    NResample <- 100
    KAll <- SCorEstMMParams$KAll
  
    for(b in 3 : 4)
    {
      now <- proc.time()[3]
      thisObs <- Observations[[b]]
      thisTime <- Times[[b]]
      thisEstimateList <- NULL
      try(thisEstimateList <- SCorEstMM(Y = thisObs, Time = thisTime, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      CompTime[b] <- proc.time()[3] - now
      print(paste("b =", b, "simulated curves estimation is done using", CompTime[b], 'seconds'))
      if(is.null(thisEstimateList))
      {
        print(paste("b =", b, "simulated curves estimation is bad !!!!!!!!!!!!!"))
        next
      }else
      {
        EstimateList[[b]] <- thisEstimateList
      }
    }
  
    Par.Restrict <- matrix(c(0, 1, 0.5, 5), K, 4, byrow = TRUE)
    for(b in 3 : 4)
    {
      now <- proc.time()[3]
      thisTime <- Times[[b]]      
      thisEstimateList <- EstimateList[[b]]
      # ParEst.Raw <- thisEstimateList$ParFitted.Ind        
      # ParEst.Refine <- ParEst.Raw
      # Alpha1 <- ParEst.Refine[,,,1]%%360
      # Alpha2 <- Alpha1%%90
      # ParEst.Refine[,,,1] <- Alpha2        
      # ParEst.Refine[,,,2] <- ifelse(((Alpha1 - Alpha2) / 90)%%2 == 1, 1 / ParEst.Refine[,,,2], ParEst.Refine[,,,2])
      # Par.Full.Refine <- drop(apply(ParEst.Refine, MARGIN = c(2,3,4), FUN = mean, trim = 0.2))
      ParPullEst.Raw <- thisEstimateList$ParFittedPull.Ind
      ParPullEst.Refine <- ParPullEst.Raw
      Alpha1 <- ParPullEst.Refine[,,1]%%360
      Alpha2 <- Alpha1%%90
      ParPullEst.Refine[,,1] <- Alpha2
      ParPullEst.Refine[,,2] <- ifelse(((Alpha1 - Alpha2) / 90)%%2 == 1, 1 / ParPullEst.Refine[,,2], ParPullEst.Refine[,,2]) 
      ParPull.Full.Refine <- matrix(apply(ParPullEst.Refine, MARGIN = c(2,3), FUN = mean, trim = 0.2), K, 4, byrow = TRUE)      
      temp <- IsotropyTest(Par.Full = ParPull.Full.Refine, Par.Restrict = Par.Restrict, Coordinate = SCorEstMMParams$Coordinate, 
                                       K = SCorEstMMParams$KAll, EstimateList = thisEstimateList, NResample = NResample, SCorEstMMParams = SCorEstMMParams, Time = thisTime)
      TestResult[[b]] <- temp
      CompTime[b] <- proc.time()[3] - now      
      print(paste("b =", b, "simulated curve test is done using", CompTime[b], 'seconds'))    
    }

  #---------------------------------------------------------------------------------------------------------------------
  # Summarize Isotropy Test Results
  #---------------------------------------------------------------------------------------------------------------------

    AlphaObs <- c()
    AlphaResample <- matrix(NA, B, NResample)
    for(b in 1 : 4)
    {
      thisTime <- Times[[b]]      
      thisEstimateList <- EstimateList[[b]]
      AlphaEstPull.Raw <- thisEstimateList$ParFittedPull.Ind[,1,1]%%90
      AlphaEstPull.Raw <- ifelse(AlphaEstPull.Raw > 45, AlphaEstPull.Raw - 90, AlphaEstPull.Raw)
      AlphaObs[b] <- mean(AlphaEstPull.Raw, trim = 0.2)      
      AlphaEstPullBoot.Raw <- TestResult[[b]]$BootAlphaPullEst.UnRotate%%90
      AlphaEstPullBoot.Raw <- ifelse(AlphaEstPullBoot.Raw > 45, AlphaEstPullBoot.Raw - 90, AlphaEstPullBoot.Raw)
      AlphaResample[b,] <- apply(AlphaEstPullBoot.Raw, MARGIN = 1, FUN = mean, trim = 0.2)
    }
   
    save.image(paste(corename, ".RData", sep = ""))
  
  #---------------------------------------------------------------------------------------------------------------------
  # Plot Results
  #---------------------------------------------------------------------------------------------------------------------
 
    pdf(paste(corename, ".pdf", sep = ""), width = 14, height = 9)
    par(mfcol = c(2,2))
    for(b in 1 : 4)
    {
      hist(AlphaResample[b,], xlim = c(-45, 45), xaxp = c(-45, 45, 6), main = "")
      Critical <- quantile(AlphaResample[b,], probs = c(0.025, 0.975))
      abline(v = Critical, col = 2, lwd = 3)
      abline(v = AlphaObs[b], col = 3, lwd = 3)
      title("Pulled estimate of Alpha")
    }
    dev.off()
  
  