


						###########################################################
						####   explore fPC scores of Harvard forest data set   ####
						###########################################################


	rm(list = ls())
	workingpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change"
	graphpath <- "X:/Research/Computations/R_WORK/Functional Data Analysis On Climate Change/graphs"
	setwd(workingpath)

	#---------------------------------------------------------------------------------------------------------------------
	# load libraries and functions
	#---------------------------------------------------------------------------------------------------------------------

		library(fda)
		library(fields)	
		library(gplots)
		library(scatterplot3d)
		library(vrmlgen)
		library(corpcor)
		library(fields)
		source("UF.repmat.r")
		source("UF.DISTgen.r")
		source("UF.fdaFIT.r")
		source("UF.fdapfc_v2.r")
    source("UF.pfceig.r") 
            
	#---------------------------------------------------------------------------------------------------------------------
	# fit EVI data
	#---------------------------------------------------------------------------------------------------------------------
  
    load("101005.Harvard_VI.RData")
		corename <- "131011.Harvard_fPC_scores_explore"
    threshold <- 0.1
		tempEVI <- EVIfull
		tempEVI[tempEVI < threshold] <- NA
    filter.specific <- complete.cases(tempEVI)
  	VIdata <- EVIfull[filter.specific, ]
    loglam <- NA	
		NYear <- 6
		TEnd <- NYear * 46
		timerange <- c(0, TEnd)
		time <- 1 : TEnd - 0.5
    list423.FU <- fdaFIT(VIdata, time = time, timerange = timerange, BASname = "F", method = "batch", penalty = 2, range = c(-3, 3), step = 0.1, loglam = loglam, show = TRUE)
    fit423.FU <- list423.FU$fit$fd						
	  ppc423.FU <- fdapfc_v2(fit423.FU, feature = "periodic", frequency = NYear, m = 46, center.fit = TRUE, center.pc = TRUE)            
	
  #---------------------------------------------------------------------------------------------------------------------
	# compute scores
	#--------------------------------------------------------------------------------------------------------------------
	
	# prepare EVI data and coordinates

		EVIfull <- data.matrix(EVIfull)
		filter.impute <- !filter.specific # index of pixels need to be imputed
		EVIimpute <- EVIfull[filter.impute, ]
		EVIimpute[EVIimpute < 0.1] <- NA # observations from pixels to be imputed
		Nimpute <- sum(filter.impute)
		edge <- 25 
		idxrow <- rep(1 : edge, times = edge)
		idxcol <- rep(1 : edge, each = edge)
		idx <- cbind(idxrow, idxcol) # pixel coordinates

	# prepare PACE ingredients

		basis <- fit423.FU$basis
		nbasis <- basis$nbasis	
		uncenterpc.list <- pca.fd(fit423.FU, nharm = nbasis - 1)
		uncenterpc <- uncenterpc.list$harmonics
		PHI <- eval.fd(time, uncenterpc) # row <-> time, col <-> fPCs
		LAMBDA <- diag(uncenterpc.list$values[-nbasis])		
		fitted <- t(eval.fd(time, fit423.FU))
		true <- data.matrix(EVI)
		res.mat <- true - fitted
		res.vec <- as.vector(res.mat)
		sigma2 <- var(res.vec) 
		SIGMA <- PHI %*% LAMBDA %*% t(PHI) + sigma2 * diag(nbasis - 1) # covariance matrix at full time grids
		meanfd <- mean.fd(fit423.FU)
		meaneval <- eval.fd(time, meanfd)	

	# perform PACE estimate

		imputepcscore <- array(NA, dim = c(nbasis - 1, Nimpute))
		for(i in 1 : Nimpute)
		{
			thisevi <- EVIimpute[i, ] # observation of the ith pixel to be imputed
			thisnonmissing <- !is.na(thisevi) # non missing time index
			thisobs <- thisevi[thisnonmissing] # observations without missingness
			thismean <- meaneval[thisnonmissing] # mean evaluation at nonmissing time index
			thisPHI <- PHI[thisnonmissing, ] # evaluation of the eigenfunctions at nonmissing time index
			thisSIGMA <- SIGMA[thisnonmissing, thisnonmissing] # evaluation covariance at nonmissing time index
			imputepcscore[ ,i] <- LAMBDA %*% t(thisPHI) %*% pseudoinverse(thisSIGMA) %*% as.vector(thisobs - thismean) # imputed eigen scores 
			print(paste("pixel", i, "is estimated"))
		}

	# construct imputed curves

		fdimpute.coef <- uncenterpc$coefs %*% imputepcscore + repmat(meanfd$coefs, 1, Nimpute) # basis coefficients
		fdimpute <- fd(fdimpute.coef, basis) # imputed fd objects
		fdimputecenter.coef <- fdimpute.coef # vertical centering the coefficients
		fdimputecenter.coef[1, ] <- 0 
		fdimputecenter <- fd(fdimputecenter.coef, basis) # vertically centered fd objects
		
	# construct fitted curves

		centerpc <- ppc423.FU$centered.pc # pc of vertically centered curves
		centerfit <- ppc423.FU$center.fit # centered unimputed curves 
		evifitcenter.coef <- matrix(NA, nbasis, edge^2)
		evifitcenter.coef[,filter.specific] <- centerfit$coefs # fill in the unimputed curves coefficients
		evifitcenter.coef[,filter.impute] <- fdimputecenter$coefs # fill in the imputed curves coefficients
		evifitcenter <- fd(evifitcenter.coef, basis) # entire fitted curves

	# compute scores

		fPCscore <- t(evifitcenter$coefs) %*% centerpc$coefs # projected eigen scores  
		#fPCscoremap <- matrix(fPCscore, edge, edge, byrow = FALSE)
	
  # bootstrap
    B <- 500
    bootfPCscore <- array(NA, dim = c(B, edge^2, 4))
    for(b in 1 : B)
    {
      index <- sample(1 : edge^2, edge^2, replace = TRUE)              
      bootRes <- res.vec[sample(1 : length(res.vec), edge^2, replace = TRUE)] 
      thisbootCurve <- evifitcenter[index]
      thisbootCurveEval <- eval.fd(time, thisbootCurve)
      thisbootCurveObs <- thisbootCurveEval + matrix(bootRes, 276, edge^2)
      listbootfit <- fdaFIT(data = t(thisbootCurveObs), time = time, timerange = timerange, BASname = "F", method = "batch", penalty = 2, range = c(-3, 3), step = 0.1, loglam = loglam, show = FALSE)
      thisbootfit <- listbootfit$fit$fd  	
      thisbootppclist <- fdapfc_v2(thisbootfit, feature = "periodic", frequency = NYear, m = 46, center.fit = TRUE, center.pc = TRUE)
      thisbootppc <- thisbootppclist$centered.pc
      bootfPCscore[b,,] <- t(thisbootfit$coefs) %*% thisbootppc$coefs[,1:4]
      print(paste("boot curve b = ", b, "is done"))
    }    
              
	# compute correlations by distance and northwest-southeast and northeast-southwest orientation

		lvec <- seq(0,12,2)
		L <- length(lvec)
		nbr.pair.NESW <- array(list(), dim = c(L,1))
		nbr.pair.NWSE <- array(list(), dim = c(L,1))
		for(l in 1 : L)
		{
		  thisl <- lvec[l]
		  temp <- which(distmat == thisl, arr.ind = TRUE)
		  dist.x <- coordinate.x[temp[,1]] - coordinate.x[temp[,2]]
		  dist.y <- coordinate.y[temp[,1]] - coordinate.y[temp[,2]]
		  temp.NESW <- temp[which(dist.x == dist.y),]
		  temp.NWSE <- temp[which(dist.x == -dist.y),]
		  nbr.pair.NESW[[l]] <- temp.NESW
		  nbr.pair.NWSE[[l]] <- temp.NWSE
		}
		K <- 4
	  COV.NESW.boot <- array(NA, dim = c(B,K,K,L))
		COV.NWSE.boot <- array(NA, dim = c(B,K,K,L))
		COR.NESW.boot <- array(NA, dim = c(B,K,K,L))
		COR.NWSE.boot <- array(NA, dim = c(B,K,K,L))
		COR.NESW.diag.boot <- array(NA, dim = c(B,K,L))
	  COR.NWSE.diag.boot <- array(NA, dim = c(B,K,L))
		COV.NESW <- array(NA, dim = c(K,K,L))
		COV.NWSE <- array(NA, dim = c(K,K,L))
		COR.NESW <- array(NA, dim = c(K,K,L))
		COR.NWSE <- array(NA, dim = c(K,K,L))
		COR.NESW.diag <- array(NA, dim = c(K,L))
		COR.NWSE.diag <- array(NA, dim = c(K,L))
            
		for(b in 1 : B)
    {
  		for(l in 1 : L)
  		{
  			for(k in 1 : K)
  			{
  				for(j in 1 : K)
  				{
            thisfPCscore <- bootfPCscore[b,,] 
  					thisl <- lvec[l]
  					temp1.NESW <- thisfPCscore[nbr.pair.NESW[[l]][,1],j]
  					temp2.NESW <- thisfPCscore[nbr.pair.NESW[[l]][,2],k]
  					COV.NESW.boot[b,j,k,l] <- cov(temp1.NESW, temp2.NESW)
            COR.NESW.boot[b,j,k,l] <- cor(temp1.NESW, temp2.NESW)
  					temp1.NWSE <- thisfPCscore[nbr.pair.NWSE[[l]][,1],j]
  					temp2.NWSE <- thisfPCscore[nbr.pair.NWSE[[l]][,2],k]
            COV.NWSE.boot[b,j,k,l] <- cov(temp1.NWSE, temp2.NWSE)
            COR.NWSE.boot[b,j,k,l] <- cor(temp1.NWSE, temp2.NWSE)
            
            temp1.NESW <- fPCscore[nbr.pair.NESW[[l]][,1],j]
            temp2.NESW <- fPCscore[nbr.pair.NESW[[l]][,2],k]
            COV.NESW[j,k,l] <- cov(temp1.NESW, temp2.NESW)
            COR.NESW[j,k,l] <- cor(temp1.NESW, temp2.NESW)
            temp1.NWSE <- fPCscore[nbr.pair.NWSE[[l]][,1],j]
            temp2.NWSE <- fPCscore[nbr.pair.NWSE[[l]][,2],k]
            COV.NWSE[j,k,l] <- cov(temp1.NWSE, temp2.NWSE)
            COR.NWSE[j,k,l] <- cor(temp1.NWSE, temp2.NWSE)            
  				} 
  			}
  			COR.NESW.diag.boot[b,,l] <- diag(COR.NESW.boot[b,,,l])
  			COR.NWSE.diag.boot[b,,l] <- diag(COR.NWSE.boot[b,,,l])
  			COR.NESW.diag[,l] <- diag(COR.NESW[,,l])
  			COR.NWSE.diag[,l] <- diag(COR.NWSE[,,l])
  		}
  		print(paste("boot curve b = ", b, "is done"))
    }

	save.image(paste(corename,".RData",sep = ""))

	# plot results
			
		pdf(paste(corename,".pdf",sep = ""), width = 14, height = 9)

			par(mfrow = c(2,2))
			for(k in 1 : K)
			{
  			  plot(lvec, COR.NESW.diag.boot[1,k,1:7], ylim = c(-0.1,1), type = "l", lty = 1, lwd=0.5, col = "grey60")
			    for(b in 1 : B)
          {
            lines(lvec, COR.NESW.diag.boot[b,k,1:7], ylim = c(-0.1,1), lty = 1, lwd=0.5, col = "grey60")
            lines(lvec, COR.NWSE.diag.boot[b,k,1:7], ylim = c(-0.1,1), lty = 1, lwd=0.5, col = "grey20")            
          }
  			  points(lvec, COR.NESW.diag[k,1:7], ylim = c(-0.1,1), col = 2, lwd=2, pch = 19, ylab = "correlation", xlab = "distance", main = paste("fPC",k," score",sep = ""),cex = 2)  			  
          lines(lvec, COR.NESW.diag[k,1:7], ylim = c(-0.1,1), lty = 1, lwd=2, col = 2, ylab = "correlation", xlab = "distance", main = paste("fPC",k," score",sep = ""),cex = 2)
  			  points(lvec, COR.NWSE.diag[k,1:7], ylim = c(-0.1,1), col = 3, lwd=2, pch = 19, ylab = "correlation", xlab = "distance", main = paste("fPC",k," score",sep = ""),cex = 2)
  			  lines(lvec, COR.NWSE.diag[k,1:7], ylim = c(-0.1,1), lty = 1, lwd=2, col = 3, ylab = "correlation", xlab = "distance", main = paste("fPC",k," score",sep = ""),cex = 2)
  			  legend("topright", lwd = c(2,2), pch = c(19,19), col = c(2,3), legend = c("NESW","NWSE"), bty = "n")
			}

		dev.off()

		
