
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#
###################################################################
####    Function of Computing Anisotropy Matern Correlation    ####
###################################################################
####      version 3.0 Created by Chong Liu on 2012/11/21       ####
###################################################################
#### changes of version 2.0 from version 1.0                   ####
#### 1. Add Log to each individual parameters                  ####
###################################################################
#### changes from version 2.0                                  ####
#### 1. Add input Coordinate                                   ####
###################################################################
#
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
# Input
# 1.Alpha:        Angle parameter used to rotate axes in anisotropy matern correlation, in degree.
# 2.Delta:        Rescale parameter used in anisotropy matern correlation; rotated x-axis coordinate is enlarged by sqrt(Delta) while rotated y-axis coordinate is 
#                 enlarged by 1/sqrt(Delta).
# 3.Kappa:        Kappa parameter in usual matern model.
# 4.Phi:          Phi parameter in usual matern model.
# 5.Coordinate:   Cartesian cooridnates of locations
# 6.SpaSep:       A matrix of 2 columns in which each row is a spatial separation vector.
# 7.SepType:      The type of SpaSep;
#                 "cartesian" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in x-y cartesian system
#                 "polar" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in the polar system, where 1st column is rho and 2nd column is 
#                 angle in degrees.
# 8.Log:          A vector of length 4, which stores logical values indicating whether Alpha,Delta,Kappa,Phi are values after taking logarithm 
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Output
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Corr:           Anisotropy matern correlation vectors corresponding to each spatial separation vector given in SpaSep
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  AnisoMatern <- function(Alpha, Delta, Kappa, Phi, Coordinate = NULL, SpaSep = NULL, SepType = NULL, Log)
  {
    # include packages and functions
    
      # source("DistGen.r")
      # library(geoR)
    
    # prepare variables
    
      Log <- as.logical(Log)
      if(Log[1]){Alpha <- exp(Alpha)}
      if(Log[2]){Delta <- exp(Delta)}
      if(Log[3]){Kappa <- exp(Kappa)}
      if(Log[4]){Phi <- exp(Phi)}
    
    # compute mahalanobis distance
    
      if(!is.null(Coordinate))
      {
        Dist <- DistGen(Coordinate = Coordinate, DistType = "mahalanobis", Alpha = Alpha, Delta = Delta)
      }else
      {
        if(is.vector(SpaSep))
        {
          SpaSep <- t(as.matrix(SpaSep))
        }
        Dist <- DistGen(SpaSep = SpaSep, SepType = SepType, DistType = "mahalanobis", Alpha = Alpha, Delta = Delta)
      }
    # compute correlation 
      
      Corr <- matern(u = Dist, phi = Phi, kappa = Kappa)
      
    # return Corr
      
      return(Corr)
  }