  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ###################################################################
  ####               Function to Wrap AnisoMatern               ####
  ###################################################################
  ####      version 2.0 Created by Chong Liu on 2012/11/13       ####
  ###################################################################
  #### Changes from version 1.0                                  ####
  #### 1. Add Log to each individual parameters                  ####
  ###################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input
  # 1.Parameters:   A vector of undeternined length, which stores the unfixed parameters, designed for optimization where some parameters are fixed
  # 2.Alpha:        Angle parameter used to rotate axes in anisotropy matern correlation, in degree.
  # 3.Delta:        Rescale parameter used in anisotropy matern correlation; rotated x-axis coordinate is enlarged by sqrt(Delta) while rotated y-axis coordinate is 
  #                 enlarged by 1/sqrt(Delta).
  # 4.Kappa:        Kappa parameter in usual matern model.
  # 5.Phi:          Phi parameter in usual matern model.
  # 6.SpaSep:       A matrix of 2 columns in which each row is a spatial separation vector.
  # 7.SepType:      The type of SpaSep;
  #                 "cartesian" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in x-y cartesian system
  #                 "polar" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in the polar system, where 1st column is rho and 2nd column is 
  #                 angle in degrees.
  # 8.Log:          A vector of length 4, which stores logical values indicating whether Alpha,Delta,Kappa,Phi are values after taking logarithm 
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Loss:           Anisotropy matern correlation vectors corresponding to each spatial separation vector given in SpaSep
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  AnisoMaternWrap <- function(Parameters, Alpha = NA, Delta = NA, Kappa = NA, Phi = NA, SpaSep, SepType, Log)
  {
    # include packages and functions
    
      # source("AnisoMatern.r")
    
    # reset parameters
    
      ParVec <- c(Alpha, Delta, Kappa, Phi)
      ParVec[is.na(ParVec)] <- Parameters # fill in unfixed parameters
      
    # if Log?
      
      Log <- as.logical(Log)
      ParVec <- ifelse(Log, exp(ParVec), ParVec)
      Alpha <- ParVec[1]
      Delta <- ParVec[2]
      Kappa <- ParVec[3]
      Phi <- ParVec[4]
      
    # compute correlation 
      
      Corr <- AnisoMatern(Alpha = Alpha, Delta=  Delta, Kappa = Kappa, Phi = Phi, SpaSep = SpaSep, SepType = SepType, Log = c(0,0,0,0))      
      
    # return Corr
      
      return(Corr)
  }