
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##############################################################
  ####    Function to Generate Distances among Locations    ####
  ##############################################################
  ####    version 1.0 Created by Chong Liu on 2012/10/29    ####
  ##############################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.Coordinate:   Cartesian cooridnates of locations
  # 2.SpaSep:       A matrix of 2 columns which stores spatial separation vector. The first column is the separation in x direction 
  #                 and the second column is the separation in y direction.
  # 3.SepType:      Type of spatial separation vector
  #                 "cartesian" for separation in x-y plane
  #                 "polar" for separation in polar system
  # 4.DistType:     Possible choices for distance type: "euclidean","manhattan","mahalanobis".
  # 5.Alpha:        Angle (in degrees) for axes rotation in "mahalanobis" distance
  # 6.Delta:        Rescaling amount of axes in "mahalanobis" distance
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.DistMat:      The distance matrix of size Ncurve by Ncurve
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

	DistGen <- function(Coordinate = NULL, SpaSep = NULL, SepType = NULL, DistType, Alpha = 0, Delta = 1)
	{		

	  # load packages and functions

      #source("U.RepMat.r")
	  
    # generate distance matrix
      
      if(!is.null(SpaSep))
      {
        SepType <- tolower(SepType)
        if(SepType == "cartesian")
        {
          dist.x <- SpaSep[,1]
          dist.y <- SpaSep[,2]
        }else
        {
          dist.x <- SpaSep[,1] * cos(SpaSep[,2]) # SpaSep[,1] is rho and SpaSep[,2] is angle
          dist.y <- SpaSep[,1] * sin(SpaSep[,2]) # SpaSep[,1] is rho and SpaSep[,2] is angle
        }
      }else
      {
        coordinate.x <- as.matrix(Coordinate[,1])
        coordinate.y <- as.matrix(Coordinate[,2])
        N <- length(coordinate.x)
		    dist.x <-  RepMat(t(coordinate.x),N,1) - RepMat(coordinate.x,1,N)
		    dist.y <- RepMat(t(coordinate.y),N,1) - RepMat(coordinate.y,1,N)
      }
      DistType <- tolower(DistType)
		  if(DistType == "manhattan"){DistMat <- abs(dist.x) + abs(dist.y)}
		  if(DistType == "euclidean"){DistMat <- sqrt(dist.x^2 + dist.y^2)}
		  if(DistType == "mahalanobis")
      {
		    Alpha <- Alpha * 2 * pi / 360 # transform from degrees into radians
		    R <- matrix(c(cos(Alpha), sin(Alpha), -sin(Alpha), cos(Alpha)), 2, 2, byrow = TRUE) # rotation matrix
		    S <- diag(c(sqrt(Delta), 1/sqrt(Delta))) # rescaling matrix
		    A <- t(R) %*% S %*% S %*% R # mahalanobis distance matrix
		    DistMat <- sqrt(dist.x^2 * A[1,1] + 2 * dist.x * dist.y * A[1,2] + dist.y^2 * A[2,2])
		  }
      
    # return the distance matrix
        
		  return(DistMat)
		
	}