  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #########################################################################
  ####    Function to Compute Conditional Expectation of fPC Scores    ####
  #########################################################################
  ####          version 1.0 Created by Chong Liu on 2012/11/14         ####
  #########################################################################
  #### change from version 1.0                                         ####
  #### 1. Allow non-separable correlation structure                    ####
  #########################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#


#--------------------------------------------------------------------------------------------------------------------------------------
# get conditional mean of alpha (random effects) and alpha*alpha' on Y and other parameters
#--------------------------------------------------------------------------------------------------------------------------------------
# ARGUMENTS 
#		Ytilde:    stacked observations (demeaned)
#   ID:  		curve ID        
#		Bmat:			stacked matrix of basis evaluation
#		THETA:		current value of eigenfunction coeffect matrix
#		OMEGA:		correlation matrix
#		LAMBDA:		current value of eigenvalues
#		sigma:		current value of noise standard deviation
#--------------------------------------------------------------------------------------------------------------------------------------
# VALUES 
#		alpha.hat:		conditional mean of alpha
#   alphaprod.hat:	conditional mean of alpha * alpha'		
#--------------------------------------------------------------------------------------------------------------------------------------
  
  GetScore <- function(Ytilde, IDtilde, TimeIndex, EigenFun, Omega = NULL, Lambda, Sigma, Independent)
  {
    
    ID <- unique(IDtilde) # unique IDs for all curves involved
    NCurve <- length(ID) # number of curves involved
    K <- length(Lambda) # number of eigenvalues
    
    if(!Independent)
    {
      # construct PHItilde'PHItilde ( A ) and PHItildeYtilde ( B )    
       
        A <- matrix(0, K * NCurve, K * NCurve) # 
        B <- matrix(0, K * NCurve, 1)
        for(c in 1 : NCurve)
        {
          id <- ID[c] # current curve ID
          thisIdx <- TimeIndex[IDtilde == id]
          thisPHI <- EigenFun[thisIdx,]
          idx <- ((c-1)*K+1) : (c*K)
          A[idx,idx] <- t(thisPHI) %*% thisPHI
          B[idx] <- t(thisPHI) %*% Ytilde[IDtilde == id] 
        }
      
      # compute conditional mean
      
        Gamma <- matrix(0, NCurve*K, NCurve*K)
        for(k in 1 : K)
        {
          flesh <- matrix(0,K,K)
          flesh[k,k] <- Lambda[k]
          Gamma <- Gamma + kronecker(Omega[,,k], flesh)
        }
        GammaInv <- solve(Gamma)
        ScoreVec <- solve(Sigma^2 * GammaInv + A) %*% B
        ScoreMat <- matrix(ScoreVec, NCurve, K, byrow = TRUE)
    }else
    {
      ScoreMat <- matrix(NA, NCurve, K)
      for(c in 1 : NCurve)
      {
        id <- ID[c] # current curve ID
        thisIdx <- TimeIndex[IDtilde == id]
        thisPHI <- EigenFun[thisIdx,]    
        A <- t(thisPHI) %*% thisPHI
        B <- t(thisPHI) %*% Ytilde[IDtilde == id] 
        ScoreVec <- solve(Sigma^2 * diag(Lambda^(-1)) + A) %*% B
        ScoreMat[c,] <- ScoreVec
      }
    }
    
    # return scores
  
      return(ScoreMat)
  }
  
  
  
  