
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#
###################################################################
####         Function to Compute Kullback Leibler Loss         ####
###################################################################
####      version 1.0 Created by Chong Liu on 2013/06/29       ####
###################################################################
#
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
# Input:
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.   
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Output
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  KLLoss <- function(DataMat, TimeIndex, FitMu, EigenFun, Omega = NULL, Lambda, Sigma, Independent = FALSE, Separable)
  {

    # prepare variables
    
      IDtilde <- as.matrix(DataMat[,1]) # stacked ID list for each observation of the curves
      Ytilde <- as.matrix(DataMat[,2]) # stacked Y list for each observation of the curves
      YtildeDemean <- Ytilde - FitMu # stacked demeaned Y list for each observation of the curves
      ID <- unique(IDtilde) # unique IDs for all curves
      NCurve <- length(ID) # number of curves involved 
    
    # compute KL loss
      
      error <- c()
      if(Independent)
      {
        for(c in 1 : NCurve)
        {
          id <- ID[c] # this curve ID
          Idx <- which(IDtilde == id) # index of this curve in the data matrix
          thisEigenFun <- EigenFun[TimeIndex[Idx],] # eigenfunction evaluated at time points of this curve
          thisYDemean <- as.matrix(YtildeDemean[Idx]) # observation of current curve
          # thisCOV <- thisEigenFun %*% diag(Lambda) %*% t(thisEigenFun) + diag(Sigma^2, length(Idx), length(Idx))
          InvThisCov <- Sigma^(-2) * (diag(1, length(Idx), length(Idx)) - thisEigenFun %*% solve(Sigma^(2) * diag(Lambda^(-1)) + t(thisEigenFun) %*% thisEigenFun) %*% t(thisEigenFun))
          error[c] <- t(thisYDemean) %*% InvThisCov %*% thisYDemean - log(det(InvThisCov)) 
        }
      }else
      {
        if(Separable)
        {
          for(c in 1 : NCurve)
          {
              
          }
        }
      }
      
    # return results
      
      results <- list(error = error)
      return(results)      
  }