  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##############################################################################
  ####    Function to Evaluate Gaussian Kernel in Local Linear Smoothing    ####
  ##############################################################################
  ####            version 1.0 Created by Chong Liu on 2012/10/31            ####
  ##############################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.X:                Design matrix of size N by D
  # 2.X0:               Evaluation points matrix of size N0 by D
  # 3.H:                Bandwidth. Scalar for 1-dim kernel and vector of length 2 for 2-dim kernel
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.Evaluation:       The kernel evaluation matrix of size N0 by N (number of X0 by number of X)
  #                     
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Details
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # The 1-dim kernel is exp(-0.5*(X-X0)^2/H^2)/sqrt(2pi);The 2-dim kernel is exp(-0.5*((X[1]-X0[1])^2/H[1]^2+(X[2]-X0[2])^2/H[2]^2))/2pi
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  
	KerEval <- function(X, X0, H)
	{
    # load packages and functions
    
      #source("U.RepMat.r")
      
    # check data consistency
    
      if(!is.matrix(X) | !is.matrix(X0)){stop("Evaluation points must be in matrix format for both 1-dim and 2-dim kernel")}
      D <- dim(X)[2]
      N <- dim(X)[1]
      N0 <- dim(X0)[1]
      D0 <- dim(X0)[2]
		  if(D > 2){stop("keval can only handle two dimensional kernels")}
      if(D != D0){stop("The second dimension of X and X0 must be the same")}
      if(D != length(H)){stop("Bandwidth length is not consistent with data dimension")}
		
    # evaluate the kernel at X0 given X

		  if(D == 1)
		  {
		    X.mat <- rep(1, N0) %o% as.vector(X) 
			  X0.mat <- as.vector(X0) %o% rep(1, N)
			  Evaluation <- exp(-0.5*(X.mat - X0.mat)^2/(H^2))/sqrt(2*pi) 
		  }
      
		  if(D == 2)
		  {
			  X.mat1 <- rep(1, N0) %o% X[,1] 
			  X0.mat1 <- X0[,1] %o% rep(1, N)	
			  X.mat2 <- rep(1, N0) %o% X[,2] 
			  X0.mat2 <- X0[,2] %o% rep(1, N)		
			  Evaluation <- exp(-0.5*((X.mat1 - X0.mat1)^2/H[1]^2 + (X.mat2 - X0.mat2)^2/H[2]^2))/(2*pi)
		  }
    
    # return kernel evaluation

		  return(Evaluation)
	}

	