  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##########################################################################################################
  ####    Function of Computing Square Error Loss between Estimated Correlation and True Correlation    ####
  ##########################################################################################################
  ####                          version 2.0 Created by Chong Liu on 2012/11/13                          ####
  ##########################################################################################################
  #### Changes from version 1.0                                                                         ####
  #### 1. Add Log to each individual parameters                                                         ####
  ##########################################################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input
  # 1.Alpha:        Angle parameter used to rotate axes in anisotropy matern correlation, in degree.
  # 2.Delta:        Rescale parameter used in anisotropy matern correlation; rotated x-axis coordinate is enlarged by sqrt(Delta) while rotated y-axis coordinate is 
  #                 enlarged by 1/sqrt(Delta).
  # 3.Kappa:        Kappa parameter in usual matern model.
  # 4.Phi:          Phi parameter in usual matern model.
  # 5.SpaSep:       A matrix of 2 columns in which each row is a spatial separation vector.
  # 6.SepType:      The type of SpaSep;
  #                 "cartesian" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in x-y cartesian system
  #                 "polar" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in the polar system, where 1st column is rho and 2nd column is 
  #                 angle in degrees.
  # 7.Estimates:    The observed or estimated correlation corresponding to each spatial separation vector in SpaSep.
  # 8.Log:          A vector of length 4, which stores logical values indicating whether Alpha,Delta,Kappa,Phi are values after taking logarithm 
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Loss:           The sum of square difference between model correlations and estimated correlations
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  MaternLoss <- function(Alpha, Delta, Kappa, Phi, SpaSep, SepType, Estimates, Log)
  {
    # include packages and functions
    
      # source("AnisoMatern.r")
    
    # compute true model correlation
    
      Corr <- AnisoMatern(Alpha = Alpha, Delta = Delta, Kappa = Kappa, Phi = Phi, SpaSep = SpaSep, SepType = SepType, Log = Log)
      
    # compute square error loss
      
      Loss <- sum((Corr - Estimates)^2)
      
    # return loss
      
      return(Loss)
  }