  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##################################################################
  ####              Function of Wrap MaternLoss                 ####
  ##################################################################
  ####      version 2.0 Created by Chong Liu on 2012/11/13      ####
  ##################################################################
  #### Changes from version 1.0                                 ####
  #### 1. Add Log to each individual parameters                 ####
  ##################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input
  # 1.Parameters:   A vector of undeternined length, which stores the unfixed parameters, designed for optimization where some parameters are fixed.
  # 2.Alpha:        Angle parameter used to rotate axes in anisotropy matern correlation, in degree.
  # 3.Delta:        Rescale parameter used in anisotropy matern correlation; rotated x-axis coordinate is enlarged by sqrt(Delta) while rotated y-axis coordinate is 
  #                 enlarged by 1/sqrt(Delta).
  # 4.Kappa:        Kappa parameter in usual matern model.
  # 5.Phi:          Phi parameter in usual matern model.
  # 6.SpaSep:       A matrix of 2 columns in which each row is a spatial separation vector.
  # 7.SepType:      The type of SpaSep;
  #                 "cartesian" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in x-y cartesian system
  #                 "polar" indicates that the 2 columns of SpaSep are cartesian spatial separation vectors represented in the polar system, where 1st column is rho and 2nd column is 
  #                 angle in degrees.
  # 8.Estimates:    The observed or estimated correlation corresponding to each spatial separation vector in SpaSep.
  # 9.Log:          A vector of length 4, which stores logical values indicating whether Alpha,Delta,Kappa,Phi are values after taking logarithm 
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Loss:           The sum of square difference between model correlations and estimated correlations
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  MaternLossWrap <- function(Parameters, Alpha = NA, Delta = NA, Kappa = NA, Phi = NA, SpaSep, SepType, Estimates, Log)
  {
    # include packages and functions
    
      # source("MaternLoss.r")
    
    # reset parameters
    
      ParVec <- c(Alpha, Delta, Kappa, Phi)
      ParVec[is.na(ParVec)] <- Parameters # fill in unfixed parameters
      Alpha <- ParVec[1]
      Delta <- ParVec[2]
      Kappa <- ParVec[3]
      Phi <- ParVec[4]  
    
    # compute loss
    
      Loss <- MaternLoss(Alpha= Alpha, Delta = Delta, Kappa = Kappa, Phi = Phi, SpaSep = SpaSep, SepType = SepType, Estimates = Estimates, Log = Log)
      
    # return loss
      
      return(Loss)
  }