
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#
############################################################################################################
####    Function to Reconstruct Curves based on Eigenfunctions, Eigenvalues and Spatial Correlations    ####
############################################################################################################
####                          version 2.0 Created by Chong Liu on 2012/11/21                            ####
############################################################################################################
#### changes from version 1.0                                                                           ####
#### 1. Use version 2.0 GetScore where non-separable structure is allowed                               ####
############################################################################################################
#
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
# Input:
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.   
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Output
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  ReConstruct <- function(DataMat, TimeIndex, FitMu, EigenFun, Omega = NULL, Lambda, Sigma, FitMu.Regular = NULL, Independent = FALSE, KLLoss = FALSE)
  {

    # prepare variables
    
      IDtilde <- as.matrix(DataMat[,1]) # stacked ID list for each observation of the curves
      Ytilde <- as.matrix(DataMat[,2]) # stacked Y list for each observation of the curves
      YtildeDemean <- Ytilde - FitMu # stacked demeaned Y list for each observation of the curves
      ID <- unique(IDtilde) # unique IDs for all curves
      NCurve <- length(ID) # number of curves involved 
    
    # estimate fPC scores
    
      ScoreMat <- GetScore(Ytilde = YtildeDemean, IDtilde = IDtilde, TimeIndex = TimeIndex, EigenFun = EigenFun, Omega = Omega, Lambda = Lambda, Sigma = Sigma, Independent = Independent)
    
    # reconstruct curves at observed points
    
      error <- c()
      Reconstruction <- list()
      ErrorList <- list()
      for(c in 1 : NCurve)
      {
        id <- ID[c] # this curve ID
        Idx <- which(IDtilde == id) # index of this curve in the data matrix
        thisEigenFun <- EigenFun[TimeIndex[Idx],] # eigenfunction evaluated at time points of this curve
        thisY <- Ytilde[Idx] # observation of current curve
        thisYEst <- thisEigenFun %*% as.matrix(ScoreMat[c,]) + as.matrix(FitMu[Idx]) # reconstructed curve
        Reconstruction[[c]] <- thisYEst # reconstruction of original curve
        ErrorList[[c]] <- thisY - thisYEst
        error[c] <- sum((ErrorList[[c]])^2)        
      }
    
    # reconstruct curves at regular time points
      
      if(!is.null(FitMu.Regular))
      {
        FitMu.Regular <- as.matrix(FitMu.Regular)
        Reconstruction.Regular  <- RepMat(FitMu.Regular, 1, NCurve) + EigenFun %*% t(ScoreMat)
      }else
      {
        Reconstruction.Regular <- NULL
      }
      
    # return results
      
      results <- list(error = error, Reconstruction = Reconstruction, ScoreMat = ScoreMat, ErrorList = ErrorList, Reconstruction.Regular = Reconstruction.Regular)
      return(results)      
  }