
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
#
############################################################################################################
####    Function to Reconstruct Curves based on Eigenfunctions, Eigenvalues and Spatial Correlations    ####
############################################################################################################
####                          version 2.0 Created by Chong Liu on 2012/11/21                            ####
############################################################################################################
#### changes from version 1.0                                                                           ####
#### 1. Use version 2.0 GetScore where non-separable structure is allowed                               ####
############################################################################################################
#
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
# Input:
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.   
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# Output
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1.
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  ReConstruct2 <- function(DataMatTest, TimeIndexTest, FitMuTest, EigenFun, Omega, Lambda, Sigma)
  {
    # include packages and functions
    
    # prepare variables
      
      IDtilde <- as.matrix(DataMatTest[,1]) # stacked ID list for each observation of the test curves
      Ytilde <- as.matrix(DataMatTest[,2]) - as.matrix(FitMuTest) # demeaned and stacked Y list for each observation of the test curves
      ID <- unique(IDtilde) # unique IDs for all test curves
      NCurve <- length(ID) # number of curves involved in test set
      
    # estimate fPC scores
    
      ScoreMat <- GetScore(Ytilde = Ytilde, IDtilde = IDtilde, TimeIndex = TimeIndexTest, EigenFun = EigenFun, Omega = Omega, Lambda = Lambda, Sigma = Sigma)
      
    # reconstruct loss
      
      error <- c()
      for(c in 1 : NCurve)
      {
        id <- ID[c]
        Idx <- which(IDtilde == id)
        thisEigenFun <- EigenFun[TimeIndexTest[Idx],]
        thisY <- Ytilde[Idx]
        thisYEst <- thisEigenFun %*% as.matrix(ScoreMat[c,])
        error[c] <- sum((thisY - thisYEst)^2)        
      }
      
    # return total error
      
      return(sum(error))      
  }