  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##############################################################################
  ####    Function to Compute Cross-Validation Score for Model Selection    ####
  ##############################################################################
  ####            version 1.0 Created by Chong Liu on 2012/11/14            ####
  ##############################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.Train 
  # 2.Test
  # 3.Coordinate
  # 4.DataMat
  # 5.TargetAll
  # 6.TargetType
  # 7.TargetIdxList
  # 8.EvalTime.Mu
  # 9.KAll
  # 10.EvalTime.Diag
  # 11.Rotate
  # 12.ReConstructType
  # 13.DefaultPars
  # 14.InitialPars
  # 15.ParsBound
  # 16.TargetRep
  # 17.Optim.Method
  # 18.Log
  # 19.MaxIt
  # 20.NGridsSearch
  # 21.Sigma
  # 22.TimeIndex
  # 23.FitMu
  # 24.CVMean
  # 25.SigmaMethod
  # 26.LLQParms
  # 27.BWMethod
  # 28.DF
  # 29.Verbose
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.Optimal
  # 2.Err.Ind
  # 3.Err.Proj
  # 4.ParFitted.Ind
  # 5.ParFitted.Proj
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

    SCorEstMMCV <- function(Train, Test, Coordinate, DataMat, TargetAll, TargetType, TargetIdxList, EvalTime.Mu, KAll, EvalTime.Diag, Rotate, ReConstructType,
                            DefaultPars, InitialPars = NULL, ParsBound = NULL, TargetRep, Optim.Method, Log, MaxIt = NULL, NGridsSearch = NULL, Sigma, TimeIndex, FitMu = NULL, 
                            CVMean = NULL, SigmaMethod = "PACE", LLQParams = NULL, BWMethod, DF, Verbose)
    {
      # load packages and functions
      
        # source("SmoothCore.r")
        # source("ReConstruct.r")
        # source("NeighborGen.r")
        # source("AnisoMaternFit.r")
        # source("AnisoMatern.r")
      
      # doing cross validation
        
        NFold <- length(Train) # number of folds
        NTarCan <- length(TargetIdxList) # number of candidate target vectors
        NK <- length(KAll) # number of candidate K
        KMax <- max(KAll) # largest candidate K
        NTask <- length(Optim.Method) # number of optimization methods
        NMaxTarget <- dim(TargetAll)[1] # max target depth
        IDList <- as.matrix(DataMat[,1]) # IDs stacked for each observation
        Err.Ind <- array(NA, dim = c(NFold, NTarCan, NK, NTask))
        Err.Proj <- array(NA, dim = c(NFold, NTarCan, NK, NTask))
        ParFitted.Ind <- array(NA, dim = c(NFold, NTarCan, KMax, NTask, 4)) # Parameter estimates container
        ParFitted.Proj <- array(NA, dim = c(NFold, NTarCan, KMax, NTask, 4)) # Parameter estimates container
        ReConstructType <- tolower(ReConstructType)
        
        for(fold in 1 : NFold)# loop through CV folds
        {
          TrainID <- Train[[fold]] # current train set
          TestID <- Test[[fold]] # current test set
          TestIdx <- sort(which(is.element(IDList,TestID))) # index corresponding to observation on test set curves outof IDList
          DataMatTest <- DataMat[TestIdx,] # extract test part of the DataMat
          TimeIndexTest <- TimeIndex[TestIdx] # time index of the test set, used in reconstruction
          CoorTrain <- Coordinate[TrainID,] # coordinates of the curves in training set
          CoorTest <- Coordinate[TestID,] # coordinates of the curves in test set          
          NeighborPairs <- NeighborGen(Coordinate = CoorTrain, SpaSepType = TargetType, SpaSep = TargetAll, CurveID = TrainID) # generate neighbors for max Target depth on training curves
          if(CVMean) 
          {
            TimeTest <- DataMatTest[,3] # time of the test set
            SmoothList <- SmoothCore(DataMat = DataMat, Curves = TrainID, NeighborPairs = NeighborPairs, EvalTime.Mu = EvalTime.Mu, EvalTime2.Mu = TimeTest, K = KMax, EvalTime.Diag = EvalTime.Diag, 
                                     Rotate = Rotate, LLQParams = LLQParams, SigmaMethod = SigmaMethod, BWMethod = BWMethod, DF = DF, Verbose = Verbose)            
            FitMuTest <- SmoothList$FitMu.Eval2 # the mean curve at test time is estimated with training set for each fold
          }else
          {
            SmoothList <- SmoothCore(DataMat = DataMat, Curves = TrainID, NeighborPairs = NeighborPairs, EvalTime.Mu = EvalTime.Mu, K = KMax, EvalTime.Diag = EvalTime.Diag, 
                                     Rotate = Rotate, LLQParams = LLQParams, SigmaMethod = SigmaMethod, BWMethod = BWMethod, DF = DF, Verbose = Verbose)
            FitMuTest <- FitMu[TestIdx] # the mean curve at test time is evaluated on one-time fitting of the mean curve with full data  
          }
          if(Rotate)
          {
            SigmaEst <- ifelse(!is.nan(SmoothList$SigmaEst.Rotate), SmoothList$SigmaEst.Rotate, Sigma)
          }else
          {
            SigmaEst <- ifelse(!is.nan(SmoothList$SigmaEst.UnRotate), SmoothList$SigmaEst.UnRotate, Sigma)            
          }
          CovFitMat <- SmoothList$CovFitMat
          XCovFitMat <- SmoothList$XCovFitMat
          EigenFunEst <- SmoothList$EigenFunEst
          LambdaEst <- SmoothList$LambdaEst
          XLambdaEst <- SmoothList$XLambdaEst
          RhoEst.Ind <- XLambdaEst / RepMat(t(LambdaEst), NMaxTarget, 1) # ratio of individual eigenvalues         
            
          for(t in 1 : NTarCan) # loop through Target candidates
          {
            thisTargetIdx <- TargetIdxList[[t]] # current target index vector
            thisTargetRep <- rbind(TargetRep[thisTargetIdx,]) # current target representative
            NTarget <- length(thisTargetIdx) # current number of neighborhood targets
            thisRhoEst.Proj <- RhoEstProj(CovFitMat, XCovFitMat = XCovFitMat[thisTargetIdx,,], K = KMax)$RhoEst.Proj # rho.proj estimates for max K eigenvalues and current targets        
            thisRhoEst.Ind <- rbind(RhoEst.Ind[thisTargetIdx,]) # rho.ind estimates for max K eigenvalues and current targets
            
            if(ReConstructType == "separable") # estimate correlation model parameters using the first eigenvalue
            {
              thisRho <- thisRhoEst.Ind[,1]
              ResultList <- AnisoMaternFit(Estimates = thisRho, DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, 
                                           TargetRep = thisTargetRep, TargetType = TargetType, Optim.Method = Optim.Method, Log = Log, MaxIt = MaxIt, 
                                           NGridsSearch = NGridsSearch)
              ParFitted.Ind[fold,t,1,,] <- ResultList$ParFitted                
              thisRho <- thisRhoEst.Proj[,1]
              ResultList <- AnisoMaternFit(Estimates = thisRho, DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, 
                                           TargetRep = thisTargetRep, TargetType = TargetType, Optim.Method = Optim.Method, Log = Log, MaxIt = MaxIt, 
                                           NGridsSearch = NGridsSearch)
              ParFitted.Proj[fold,t,1,,] <- ResultList$ParFitted
            }
            if(ReConstructType == "diag") # estimate correlation model parameters using each individual eigenvalues
            {
              for(k in 1 : KMax)
              {
                thisRho <- thisRhoEst.Ind[,k]
                ResultList <- AnisoMaternFit(Estimates = thisRho, DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, 
                                             TargetRep = thisTargetRep, TargetType = TargetType, Optim.Method = Optim.Method, Log = Log, MaxIt = MaxIt, 
                                             NGridsSearch = NGridsSearch)
                ParFitted.Ind[fold,t,k,,] <- ResultList$ParFitted                
                thisRho <- thisRhoEst.Proj[,k]
                ResultList <- AnisoMaternFit(Estimates = thisRho, DefaultPars = DefaultPars, InitialPars = InitialPars, ParsBound = ParsBound, 
                                             TargetRep = thisTargetRep, TargetType = TargetType, Optim.Method = Optim.Method, Log = Log, MaxIt = MaxIt, 
                                             NGridsSearch = NGridsSearch)
                ParFitted.Proj[fold,t,k,,] <- ResultList$ParFitted 
              }
            }
            
            for(task in 1 : NTask) # given the current target vector candidate t, loop through all optimization methods
            {
              Omega.Ind <- array(NA, dim = c(length(TestID), length(TestID), KMax))
              Omega.Proj <- array(NA, dim = c(length(TestID), length(TestID), KMax))
              if(ReConstructType == "separable") # compute correlation matrix of scores common to all eigenvalues
              {
                temp.Ind <- AnisoMatern(Coordinate = CoorTest, Alpha = ParFitted.Ind[fold,t,1,task,1], Delta = ParFitted.Ind[fold,t,1,task,2], 
                                        Kappa = ParFitted.Ind[fold,t,1,task,3], Phi = ParFitted.Ind[fold,t,1,task,4], Log = c(0,0,0,0))
                # last step computes the common correlation matrix using the current estimated model parameters
                Omega.Ind <- array(temp.Ind, dim = c(dim(temp.Ind), KMax)) # this step populate the common matrix to all eigenvalues
                temp.Proj <- AnisoMatern(Coordinate = CoorTest, Alpha = ParFitted.Proj[fold,t,1,task,1], Delta = ParFitted.Proj[fold,t,1,task,2], 
                                         Kappa = ParFitted.Proj[fold,t,1,task,3], Phi = ParFitted.Proj[fold,t,1,task,4], Log = c(0,0,0,0)) 
                Omega.Proj <- array(temp.Proj, dim = c(dim(temp.Proj), KMax))
              }              
              if(ReConstructType == "diag") # compute correlation matrix of scores for each eigenvalues
              {
                for(k in 1 : KMax)
                {
                  Omega.Ind[,,k] <- AnisoMatern(Coordinate = CoorTest, Alpha = ParFitted.Ind[fold,t,k,task,1], Delta = ParFitted.Ind[fold,t,k,task,2], 
                                                Kappa = ParFitted.Ind[fold,t,k,task,3], Phi = ParFitted.Ind[fold,t,k,task,4], Log = c(0,0,0,0))
                  Omega.Proj[,,k] <- AnisoMatern(Coordinate = CoorTest, Alpha = ParFitted.Proj[fold,t,k,task,1], Delta = ParFitted.Proj[fold,t,k,task,2], 
                                                 Kappa = ParFitted.Proj[fold,t,k,task,3], Phi = ParFitted.Proj[fold,t,k,task,4], Log = c(0,0,0,0))
                }
              }

              for(kk in 1 : NK) # loop through K candidates
              {
                thisK <- KAll[kk] # current number of eigenfunctions retained
                thisEigenFun <- as.matrix(EigenFunEst[,1:thisK]) # current eigenfunctions retained
                thisLambda <- LambdaEst[1:thisK] # current eigenvalues retained
                thisOmega.Ind <- array(Omega.Ind[,,1:thisK], dim = c(dim(Omega.Ind)[1], dim(Omega.Ind)[2], thisK)) # current score covariance array
                thisOmega.Proj <- array(Omega.Proj[,,1:thisK], dim = c(dim(Omega.Proj)[1], dim(Omega.Proj)[2], thisK)) # current score covariance array
                
                Err.Ind[fold,t,kk,task] <- sum(ReConstruct(DataMat = DataMatTest, TimeIndex = TimeIndexTest, FitMu = FitMuTest, EigenFun = thisEigenFun, 
                                                           Omega = thisOmega.Ind, Lambda = thisLambda, Sigma = SigmaEst)$error)
                Err.Proj[fold,t,kk,task] <- sum(ReConstruct(DataMat = DataMatTest, TimeIndex = TimeIndexTest, FitMu = FitMuTest, EigenFun = thisEigenFun, 
                                                            Omega = thisOmega.Proj, Lambda = thisLambda, Sigma = SigmaEst)$error)
              }
            }            
          } 
          print(paste("Cross-Validation fold", fold, "is done"))
        }
                       
      # pick the smallest error
        
        ErrMean.Ind <- apply(Err.Ind, MARGIN = c(2,3,4), FUN = mean)
        ErrMean.Proj <- apply(Err.Proj, MARGIN = c(2,3,4), FUN = mean)
        Optimal <- matrix(NA, NTask, 2)
        MyFun <- function(x){if(length(x)>1){xdiff<-diff(x);if(all(xdiff<0)){z<-which(xdiff == min(xdiff))[1]+1}else{z<-which(x == min(x))[1]}}else{z<-1};return(z)} 
        # choose optimal K for each Target to be the K which produce the largest drop of CV score if CV is monotone decreasing cross all K
        for(task in 1 : NTask)
        {
          thisErr.Ind <- ErrMean.Ind[,,task]
          thisErr.Ind <- matrix(thisErr.Ind, dim(ErrMean.Ind)[1], dim(ErrMean.Ind)[2])
          KIdxVec <- apply(thisErr.Ind, MARGIN = 1, FUN = MyFun)
          TmpErr <- thisErr.Ind[NTarCan * (KIdxVec - 1) + (1 : NTarCan)]
          TargetIdx <- which(TmpErr == min(TmpErr))[1] # choose optimal NTarget, the most parsimonious model 
          Optimal[task,] <- c(TargetIdx, KIdxVec[TargetIdx]) # optimal model selection, (NTarget, K)
        }
        
      # return results
        
        return(list(Optimal = Optimal, Err.Ind = Err.Ind, Err.Proj = Err.Proj, ParFitted.Ind = ParFitted.Ind, ParFitted.Proj = ParFitted.Proj)) 
    }
  
  
  
        
    
  
    
    
      
  
         
  
  