
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ################################################################
  ####    Function to Generate Spatially Correlated Curves    ####
  ################################################################
  ####     version 1.0 Created by Chong Liu on 2012/11/18     ####
  ################################################################
  #### change of version 2.0 from version 1.0                 ####
  #### Add output of fPC scores                               ####
  ################################################################
  #### change of version 3.0 from version 2.0                 ####
  #### 1. Remove CorType                                      ####
  ################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Let NCurve be the number of curves, K be the number of eigenfunctions, 
  #                                              
  # 1.CorMat:     array of size K by NCurve by NCurve of distance between any two pixels.
  # 2.MeanFD:     functional data object of the mean trend (fixed effect)
  # 3.EigenFun:   functional data object of K eigenfunctions                                                
  # 4.Lambda:     a vector of length K, of the K eigenvalues
  # 5.B:          number of replicated data sets
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#    
  # Output:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#    
  # 1.Curves:     Functional data object of length B
  # 2.Scores:     A matrix of size NCurve by K by B, the fPC scores of all curves and all eigenfunctions
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#    

	SCorFunGen <- function(CorMat, MeanFD, EigenFun, Lambda, B)
	{		

	  # load packages and functions

		  library(fda)
		  library(mnormt)
		  #source("U.RepMat.r")

    # construct covariance structure and eigen scroes
      
		  NCurve <- dim(CorMat)[2] # number of curves
		  K <- length(Lambda) # number of eigenfunctions(eigenvalues)			
		  Scores <- array(NA, dim = c(NCurve,K,B))
      
      for(k in 1 : K)
			{
			  thiscov <- Lambda[k] * CorMat[k,,] # covariance matrix of score k of NCurve curves
			  temp <- rmnorm(B, rep(0, NCurve), thiscov) # B rows of random vector
			  Scores[,k,] <- t(temp)
		  }	

  	# generate noiseless curves

		  Curves <- array(list(), dim = c(B))
		  basis <- EigenFun$basis
		  eigenfcoef <- EigenFun$coefs
		  meancoef <- MeanFD$coefs
		  for(b in 1 : B)
		  {
		  	thisscore <- Scores[,,b] # bth simulated score of size NCurve by K
		  	Curves[[b]] <- fd(eigenfcoef %*% t(thisscore) + RepMat(as.matrix(meancoef), 1, NCurve), basis)
		  }
      
    # return curves
      
		  return(list(Curves = Curves, Scores = Scores))		
	 }