  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #############################################################################
  ####    Function to Compute Sample Spatial Correlation between Scores    ####
  #############################################################################
  ####           version 1.0 Created by Chong Liu on 2012/11/18            ####
  #############################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input:
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.Scores:           A matrix of size B by K by NCurve by A, the fPC scores of all curves and all eigenfunctions
  # 2.Coordinate:       The usual cartesian coordinate of all locations
  # 3.CoorTargetType:   Type of coordinates used in the CoorTarget.
  #                     "cartesian" is the usual cartesian distance separation vector. The second dimension in CoorTarget correspond to x and y axis;
  #                     "polar" is the polar coordinates. The second dimension in CoorTarget correspond to distance and angle.
  # 4.CoorTarget:       A matrix of size NTarget by 2 by 2. The first dimension is the number of neighborhood in consideration(number of XCov need to be fitted). The second dimension 
  #                     is the end points of the interval which used to determine neighborhoods. The third dimension is for two separation variables a and b.
  # 5.CurveID:          IDs of curves involved
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # 1.SampleCor:        A list array of size NTarget by A, each cell being an array of size NCurve by K by K of spatial correlations
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

    SampleCor <- function(Scores, Coordinate, CoorTargetType, CoorTarget, CurveID = NULL)
    {
      # include packages and functions
      
        #source("NeighborGen.r")      
      
      # generate neighbor pairs
        
        NeighborPairs <- NeighborGen(Coordinate = Coordinate, CoorTargetType = CoorTargetType, CoorTarget = CoorTarget, CurveID = CurveID)
        
      # compute sample fPC score covariance
        
        NTarget <- length(NeighborPairs)
        A <- dim(Scores)[4]
        K <- dim(Scores)[2]
        SampleCor <- array(list(), dim = c(NTarget,A))
        for(target in 1 : NTarget)
        {
          for(a in 1 : A)
          {
            thisNeighbor <- unique(t(apply(NeighborPairs[[target]], FUN = sort, MARGIN = 1)))
            NPair <- dim(thisNeighbor)[1]
            SampleCor[[target,a]] <- array(NA, dim = c(NPair,K,K)) 
            
            for(pair in 1 : NPair)
            {
              score1 <- t(Scores[thisNeighbor[pair,1],,,a])
              score2 <- t(Scores[thisNeighbor[pair,2],,,a])
              SampleCor[[target,a]][pair,,] <- cor(score1, score2)
            }
          }
        }
        
      # return correlation
        
        return(SampleCor)
    }