  
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  ##################################################################
  ####              Function of Separability Test               ####
  ##################################################################
  ####      version 1.0 Created by Chong Liu on 2013/07/13      ####
  ##################################################################
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#                                              
  # Input
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  #
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#
  # Output
  #-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------#

  SeparabilityTest <- function(Par.Full, Par.Restrict, Coordinate, K, EstimateList, NResample, SCorEstMMParams, Time, TransformType = 3)
  {
    # Summarize Data
    
    NCurve <- dim(Coordinate)[1]
    Par.Full <- matrix(Par.Full, K, 4)
    Par.Restrict <- matrix(Par.Restrict, 1, 4)
    NT <- dim(EstimateList$ParFitted.Ind)[1]
    
    # Generate Correlation Matrix Omega
     
    Omega.Full <- array(NA, dim = c(NCurve, NCurve, K)) # score correlation array using full model with no restriction for all eigenvalues
    for(k in 1 : K)
    {
      Omega.Full[,,k] <- AnisoMatern(Coordinate = Coordinate, Alpha = Par.Full[k,1], Delta = Par.Full[k,2], Kappa = Par.Full[k,3], Phi = Par.Full[k,4], Log = c(0,0,0,0))
    }
    Omega.H0 <- AnisoMatern(Coordinate = Coordinate, Alpha = Par.Restrict[1], Delta = Par.Restrict[2], Kappa = Par.Restrict[3], Phi = Par.Restrict[4], Log = c(0,0,0,0))
    Omega.Restrict <- array(Omega.H0, dim = c(NCurve, NCurve, K)) # score correlation array using restricted model for all eigenvalues          
    
    # Estimate fPC scores using Omega.Full
    
    DataMat <- EstimateList$DataMat # current data matrix
    ID <- as.matrix(DataMat[,1]) # current stacked ID
    Lambda <- EstimateList$SmoothList$LambdaEst # current estimated lambda
    TimeIndex <- as.matrix(EstimateList$TimeIndex) # current time index
    FitMu <- as.matrix(EstimateList$SmoothList$FitMu) # curent estimated mean evaluated at all time points
    FitMuEval <- as.matrix(EstimateList$SmoothList$FitMu.Eval) # current estimated mean evaluated at EvalTime.Mu
    EigenFun <- EstimateList$SmoothList$EigenFunEst # current estimated
    Sigma.Rotate <- EstimateList$SigmaEst.Rotate # current estimated sigma with rotation
    Sigma.UnRotate <- EstimateList$SigmaEst.UnRotate # current estimated sigma without rotation
    #Reconstruction.Rotate <- ReConstruct(DataMat = DataMat, TimeIndex = TimeIndex, FitMu = FitMu, EigenFun = EigenFun, Omega = Omega.Full, Lambda = Lambda, Sigma = Sigma.Rotate)
    Reconstruction.UnRotate <- ReConstruct(DataMat = DataMat, TimeIndex = TimeIndex, FitMu = FitMu, EigenFun = EigenFun, Omega = Omega.Full, Lambda = Lambda, Sigma = Sigma.UnRotate)
    #ScoreMat.Rotate <- Reconstruction.Rotate$ScoreMat # full model score estimation with Sigma.Rotate    
    ScoreMat.UnRotate <- Reconstruction.UnRotate$ScoreMat # full model score estimation with Sigma.UnRotate          
    TransMat <- array(NA, dim = c(NCurve, NCurve, K))
    InvTransMat <- array(NA, dim = c(NCurve, NCurve, K))
    #Z.Rotate <- matrix(NA, NCurve, K)
    Z.UnRotate <- matrix(NA, NCurve, K)
    Omega.H0.Eigen <- eigen(Omega.H0)
    for(k in 1 : K)
    {
      if(TransformType == 1)
      {
        TransMat[,,k] <- Omega.H0.Eigen$vectors %*% diag((sqrt(Omega.H0.Eigen$values * Lambda[k]))^(-1)) %*% t(Omega.H0.Eigen$vectors) # matrix used to transform score to z, under restricted model, the null hypothesis
        InvTransMat[,,k] <- Omega.H0.Eigen$vectors %*% diag(sqrt(Omega.H0.Eigen$values * Lambda[k])) %*% t(Omega.H0.Eigen$vectors) # matrix used to transform z back to score, under restricted model, the null hypothesis        
      }
      if(TransformType == 2)
      {
        TransMat[,,k] <- diag((sqrt(Omega.H0.Eigen$values * Lambda[k]))^(-1)) %*% t(Omega.H0.Eigen$vectors) # matrix used to transform score to z, under restricted model, the null hypothesis
        InvTransMat[,,k] <- Omega.H0.Eigen$vectors %*% diag(sqrt(Omega.H0.Eigen$values * Lambda[k])) # matrix used to transform z back to score, under restricted model, the null hypothesis
      }
      if(TransformType == 3)
      {
        TransMat[,,k] <-  t(solve(chol(Omega.H0 * Lambda[k])))
        InvTransMat[,,k] <-  t(chol(Omega.H0 * Lambda[k]))
      }      
      #Z.Rotate[,k] <- TransMat[,,k] %*% ScoreMat.Rotate[,k] # z from score matrix with Sigma.Rotate
      Z.UnRotate[,k] <- TransMat[,,k] %*% ScoreMat.UnRotate[,k] # z from score matrix with Sigma.UnRotate
    }    
    ResampleCount <- 0
    TryCount <- 0
    NGrid <- length(FitMuEval)
    PermPhiEst.UnRotate <- array(NA, dim = c(NResample, NT, K))
    BootPhiEst.UnRotate <- array(NA, dim = c(NResample, NT, K))
    while((ResampleCount < NResample) & (TryCount < NResample * 1.2))
    {
      TryCount <- TryCount + 1
      PermIndex <- sample(1 : (K * NCurve), K * NCurve, replace = FALSE)
      BootIndex <- sample(1 : NCurve, NCurve, replace = TRUE)
      #PermZ.Rotate <- matrix(NA, NCurve, K)
      PermZ.UnRotate <- matrix(NA, NCurve, K)
      #BootZ.Rotate <- matrix(NA, NCurve, K)
      BootZ.UnRotate <- matrix(NA, NCurve, K)
      #PermScore.Rotate <- matrix(NA, NCurve, K)
      PermScore.UnRotate <- matrix(NA, NCurve, K)
      #BootScore.Rotate <- matrix(NA, NCurve, K)
      BootScore.UnRotate <- matrix(NA, NCurve, K)            
      for(k in 1 : K)
      {
        tempIdx <- (NCurve * (k - 1) + 1) : (NCurve * k)
        #PermZ.Rotate[,k] <- as.vector(Z.Rotate)[PermIndex[tempIdx]]
        PermZ.UnRotate[,k] <- as.vector(Z.UnRotate)[PermIndex[tempIdx]]
        #BootZ.Rotate[,k] <- Z.Rotate[BootIndex,k]
        BootZ.UnRotate[,k] <- Z.UnRotate[BootIndex,k]
        #PermScore.Rotate[,k] <- InvTransMat[,,k] %*% PermZ.Rotate[,k]
        PermScore.UnRotate[,k] <- InvTransMat[,,k] %*% PermZ.UnRotate[,k]  
        #BootScore.Rotate[,k] <- InvTransMat[,,k] %*% BootZ.Rotate[,k]
        BootScore.UnRotate[,k] <- InvTransMat[,,k] %*% BootZ.UnRotate[,k]   
      }      
      #PermCurveNoiseLess.Rotate <- EigenFun %*% t(PermScore.Rotate) + RepMat(FitMuEval, 1, NCurve)
      PermCurveNoiseLess.UnRotate <- EigenFun %*% t(PermScore.UnRotate) + RepMat(FitMuEval, 1, NCurve)   
      #BootCurveNoiseLess.Rotate <- EigenFun %*% t(BootScore.Rotate) + RepMat(FitMuEval, 1, NCurve)
      BootCurveNoiseLess.UnRotate <- EigenFun %*% t(BootScore.UnRotate) + RepMat(FitMuEval, 1, NCurve)      
      #PermCurveRegObs.Rotate <- PermCurveNoiseLess.Rotate + matrix(Sigma.Rotate * rnorm(NGrid * NCurve), NGrid, NCurve)
      PermCurveRegObs.UnRotate <- PermCurveNoiseLess.UnRotate + matrix(Sigma.UnRotate * rnorm(NGrid * NCurve), NGrid, NCurve)
      #BootCurveRegObs.Rotate <-  BootCurveNoiseLess.Rotate + matrix(Sigma.Rotate * rnorm(NGrid * NCurve), NGrid, NCurve)
      BootCurveRegObs.UnRotate <- BootCurveNoiseLess.UnRotate + matrix(Sigma.UnRotate * rnorm(NGrid * NCurve), NGrid, NCurve)      
      #PermCurveIRRegObs.Rotate <- list()
      PermCurveIRRegObs.UnRotate <- list() 
      #BootCurveIRRegObs.Rotate <- list()
      BootCurveIRRegObs.UnRotate <- list()        
      for(i in 1 : NCurve)
      {
        #PermCurveIRRegObs.Rotate[[i]] <- PermCurveRegObs.Rotate[TimeIndex[ID == i], i]
        PermCurveIRRegObs.UnRotate[[i]] <- PermCurveRegObs.UnRotate[TimeIndex[ID == i], i]
        #BootCurveIRRegObs.Rotate[[i]] <- BootCurveRegObs.Rotate[TimeIndex[ID == i], i]
        BootCurveIRRegObs.UnRotate[[i]] <- BootCurveRegObs.UnRotate[TimeIndex[ID == i], i]
      }      
      #PermEstiamteList.Rotate <- NULL
      PermEstiamteList.UnRotate <- NULL
      #BootEstiamteList.Rotate <- NULL
      BootEstiamteList.UnRotate <- NULL
      #try(PermEstimateList.Rotate <- SCorEstMM(Y = PermCurveIRRegObs.Rotate, Time = Time, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      try(PermEstimateList.UnRotate <- SCorEstMM(Y = PermCurveIRRegObs.UnRotate, Time = Time, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      #try(BootEstimateList.Rotate <- SCorEstMM(Y = BootCurveIRRegObs.Rotate, Time = Time, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      try(BootEstimateList.UnRotate <- SCorEstMM(Y = BootCurveIRRegObs.UnRotate, Time = Time, SCorEstMMParams = SCorEstMMParams), silent = TRUE)
      if(is.null(PermEstimateList.UnRotate) | is.null(BootEstimateList.UnRotate))
      {
        print(paste("ResampleCount =", ResampleCount + 1, "resampled curves estimation is bad !!!!!!!!!!!!!!!!"))
        next
      }else
      { 
        ResampleCount <- ResampleCount + 1            
        PermPhiEst.UnRotate[ResampleCount,,] <- PermEstimateList.UnRotate$ParFitted.Ind[,,1,4]
        BootPhiEst.UnRotate[ResampleCount,,] <- BootEstimateList.UnRotate$ParFitted.Ind[,,1,4]
        print(paste("ResampleCount =", ResampleCount, "resampled curves estimation is done !!!!!!!!!!!!!!!!"))
      }
    }
    Output <- list()
    Output$PermPhiEst.UnRotate <- PermPhiEst.UnRotate
    Output$BootPhiEst.UnRotate <- BootPhiEst.UnRotate
    return(Output)
  }