

########################################
### internal functions of SM package ###
########################################


#### interesting commands ####
#### substitute()

sm.check.data <- function (x, y = NA, weights = NA, group = NA, ...) 
{
    opt <- sm.options(list(...))
    density <- all(is.na(y))
    if (density) 
      
        X <- x
    else X <- cbind(x, y)
    if (all(is.na(weights)) | all(weights == 1)) 
        X <- cbind(X, 1)
    else {
        if (!is.na(opt$nbins) & opt$nbins != 0) 
            stop("if weights are set, nbins must be either 0 or NA")
        if (any(weights < 0 | is.na(weights))) 
            stop("negative or NA weights are meaningless")
        if (any(weights != round(weights))) {
            weights <- round(weights/min(weights[weights > 0]))
            if (opt$verbose > 0) 
                cat("Warning: weights have been rescaled to integer values\n")
        }
        X <- cbind(X, weights)
    }
    ndim <- ncol(X) - 1 - (!density)
    if (!all(is.na(group))) {
        X <- cbind(X, group)
        group.col <- ncol(X)
    }
    if (!all(is.na(opt$h.weights))) {
        X <- cbind(X, opt$h.weights)
        hw.col <- ncol(X)
    }
    if (any(is.na(X)) & opt$verbose > 0) 
        cat("missing data are removed\n")
    X <- na.omit(data.matrix(X))
    if (ndim > 2 + density) 
        stop("x has too many columns")
    weights <- as.vector(X[, ndim + (!density) + 1])
    if (!density) 
        y <- as.vector(X[, ndim + 1])
    x <- if (ndim == 1) 
        as.vector(X[, 1])
    else X[, 1:ndim]
    if (!all(is.na(group))) 
        group <- as.vector(X[, group.col])
    if (!all(is.na(opt$h.weights))) 
        opt$h.weights <- X[, hw.col]
    list(x = x, y = y, weights = weights, group = group, ndim = ndim, 
        nobs = nrow(X), density = density, options = opt)
}
#<environment: namespace:sm>

replace.na <- function (List, comp, value) 
{
    arg <- paste(substitute(List), "$", substitute(comp), sep = "")
    arg.value <- eval(parse(text = arg), parent.frame(1))
    if (any(is.na(arg.value))) {
        change <- paste(arg, "<-", deparse(substitute(value)))
        a <- eval(parse(text = change), parent.frame(1))
    }
    invisible()
}

cv.crit.reg <- function (log.h, data, structure.2d, opt) 
{
    x <- data$x
    y <- data$means
    freq <- data$x.freq
    h <- exp(log.h)
    if (is.vector(x)) 
        S <- sm.weight(x, x, h, cross = T, weights = freq, options = opt)
    if (is.matrix(x)) {
        h <- switch(structure.2d, scaled = h * data$sd, common = rep(h, 
            2), h)
        S <- sm.weight2(x, x, h, cross = T, weights = freq, options = opt)
    }
    n <- length(y)
    cv <- sum(freq * ((diag(n) - S) %*% y)^2 + data$devs)/sum(freq)
    if (opt$verbose > 1) 
        cat("h, CV: ", h, cv, "\n")
    if (is.na(cv)) 
        cv <- 1e+10
    cv
}

aicc.crit.reg <- function (log.h, data, structure.2d, opt) 
{
    x <- data$x
    y <- data$means
    freq <- data$x.freq
    h <- exp(log.h)
    if (is.vector(x)) 
        S <- sm.weight(x, x, h, weights = freq, options = opt)
    if (is.matrix(x)) {
        h <- switch(structure.2d, scaled = h * data$sd, common = rep(h, 
            2), h)
        S <- sm.weight2(x, x, h, weights = freq, options = opt)
    }
    tr.S <- sum(freq * diag(S))
    nobs <- sum(freq)
    n <- length(y)
    sig.sq <- sum(freq * ((diag(n) - S) %*% y)^2 + data$devs)/nobs
    penalty <- 1 + 2 * (tr.S + 1)/(nobs - tr.S - 2)
    aicc <- log(sig.sq) + penalty
    if (opt$verbose > 1) 
        cat("h, AIC.c: ", h, aicc, "\n")
    if (is.na(aicc)) 
        aicc <- 1e+10
    aicc
}

df.crit.reg <- function (log.h, data, structure.2d, opt) 
{
  x <- data$x
  freq <- data$x.freq
  h <- exp(log.h)
  if (is.vector(x)) 
    S <- sm.weight(x, x, h, weights = freq, options = opt)
  if (is.matrix(x)) {
    h <- switch(structure.2d, scaled = h * data$sd, common = rep(h, 2), h)
    S <- sm.weight2(x, x, h, weights = freq, options = opt)
  }
  (sum(diag(S)) - opt$df)^2
}

	sm.weight <- function (x, eval.points, h, cross = FALSE, weights = rep(1, length(x)), options = list()) 
	{
    		#if (!exists(".sm.Options")) 
    		#    stop("cannot find .sm.Options")
 
		opt <- sm.options(options)
    		replace.na(opt, hmult, 1)
    		replace.na(opt, h.weights, rep(1, length(x)))
    		replace.na(opt, poly.index, 1)
    		poly.index <- opt$poly.index
    		h.weights <- opt$h.weights
    		hmult <- opt$hmult
    		period <- opt$period[1]
    		n <- length(x)
    		ne <- length(eval.points)

    		wd <- matrix(rep(eval.points, rep(n, ne)), ncol = n, byrow = TRUE) # repeat eval.points by n columns
    		wd <- wd - matrix(rep(x, ne), ncol = n, byrow = TRUE) # repeat x by ne rows 
    		w <- matrix(rep(h.weights, ne), ncol = n, byrow = TRUE) # repeat bandwidth weight vector by ne rows

    		if (!is.na(opt$period)) 
		{
			w <- exp(cos(2 * pi * wd/period)/(h * hmult * w))
		}else
		{
	 		w <- exp(-0.5 * (wd/(h * hmult * w))^2)  # compute the Gaussian kernel evaluation
		}
	    
		w <- w * matrix(rep(weights, ne), ncol = n, byrow = TRUE) # add the weight (repeated observation) effect
	    	if (cross) 
		{
	        	diag(w) <- 0 # leave-one-point-out effect
		}
	    	if ((poly.index == 0) | (!is.na(period))) 
		{
	        	den <- w %*% rep(1, n)
	        	w <- w/matrix(rep(den, n), ncol = n)
	    	}else 
		{
	        	s0 <- w %*% rep(1, n)
	        	s1 <- (w * wd) %*% rep(1, n)
	        	s2 <- (w * wd^2) %*% rep(1, n)
	        	w <- w * (matrix(rep(s2, n), ncol = n) - wd * matrix(rep(s1, n), ncol = n))
	        	w <- w/(matrix(rep(s2, n), ncol = n) * matrix(rep(s0, n), ncol = n) - matrix(rep(s1, n), ncol = n)^2)
	    	}
	}
	
	sm.weight2 <- function (x, eval.points, h, cross = FALSE, weights = rep(1, nrow(x)), options = list()) 
	{
    		opt <- sm.options(options)
    		if (all(is.na(opt$period))) 
        	{
			opt$period <- rep(NA, 2)
		}
    		replace.na(opt, hmult, 1)
    		replace.na(opt, h.weights, rep(1, nrow(x)))
    		replace.na(opt, poly.index, 1)
    		poly.index <- opt$poly.index
    		h.weights <- opt$h.weights
    		hmult <- opt$hmult
    		n <- nrow(x)
    		ne <- nrow(eval.points)
    		wd1 <- matrix(rep(eval.points[, 1], rep(n, ne)), ncol = n, byrow = TRUE)
    		wd1 <- wd1 - matrix(rep(x[, 1], ne), ncol = n, byrow = TRUE)
    		if (("hw.eval" %in% names(opt)) & (opt$hw.eval = TRUE)) 
        	{
			hw <- matrix(rep(h.weights, n), ncol = n)
		}else
		{
			hw <- matrix(rep(h.weights, ne), ncol = n, byrow = TRUE)
		}
    		if (!is.na(opt$period[1]))
		{ 
        		w <- exp(cos(2 * pi * wd1/opt$period[1])/(h[1] * hmult * hw))
		}else
		{ 
			w <- exp(-0.5 * (wd1/(h[1] * hmult * hw))^2)
		}
    		wd2 <- matrix(rep(eval.points[, 2], rep(n, ne)), ncol = n, byrow = TRUE)
		wd2 <- wd2 - matrix(rep(x[, 2], ne), ncol = n, byrow = TRUE)
    		if (!is.na(opt$period[2]))
		{ 
        		w <- w * exp(cos(2 * pi * wd2/opt$period[2])/(h[2] * hmult * hw))
		}else
		{
 			w <- w * exp(-0.5 * (wd2/(h[2] * hmult * hw))^2)
		}
    		w <- w * matrix(rep(weights, ne), ncol = n, byrow = TRUE)
    		if (cross) 
        	{
			diag(w) <- 0
		}
    		if ((opt$poly.index == 0) | (sum(is.na(opt$period)) == 0)) 
		{
        		den <- w %*% rep(1, n)
        		w <- w/matrix(rep(den, n), ncol = n)
		}else if ((opt$poly.index == 1) & (sum(is.na(opt$period)) == 1)) 
		{
        		if (is.na(opt$period[2]))
			{ 
            		wd1 <- wd2
			}
        		s0 <- w %*% rep(1, n)
        		s1 <- (w * wd1) %*% rep(1, n)
        		s2 <- (w * wd1^2) %*% rep(1, n)
        		w <- w * (matrix(rep(s2, n), ncol = n) - wd1 * matrix(rep(s1, n), ncol = n))
        		w <- w/(matrix(rep(s2, n), ncol = n) * matrix(rep(s0, n), ncol = n) - matrix(rep(s1, n), ncol = n)^2)
    		}else 
		{
        		a11 <- w %*% rep(1, n)
        		a12 <- (w * wd1) %*% rep(1, n)
        		a13 <- (w * wd2) %*% rep(1, n)
        		a22 <- (w * wd1^2) %*% rep(1, n)
        		a23 <- (w * wd1 * wd2) %*% rep(1, n)
        		a33 <- (w * wd2^2) %*% rep(1, n)
        		d <- a22 * a33 - a23^2
        		b1 <- 1/(a11 - ((a12 * a33 - a13 * a23) * a12 + (a13 * a22 - a12 * a23) * a13)/d)
        		b2 <- (a13 * a23 - a12 * a33) * b1/d
        		b3 <- (a12 * a23 - a13 * a22) * b1/d
        		wt <- matrix(rep(b1, n), ncol = n)
        		wt <- wt + matrix(rep(b2, n), ncol = n) * wd1
        		wt <- wt + matrix(rep(b3, n), ncol = n) * wd2
        		w <- wt * w
    		}
    		w
	}


